/***************************************************************************
 *            guyum_install_component.c
 *
 *  $LastChangedDate: 2005-12-26 23:59:21 +0100 (Mo, 26 Dez 2005) $
 *  Copyright  2005  $Author: vklein $
 *  Email
 ****************************************************************************/
#include <include/guyum/gy_install.h>


/****************************************************************************
* Function declarations													
****************************************************************************/
static void guyum_install_component_class_init(GuyumInstallComponentClass *class);

static void guyum_install_component_init(	GuyumInstallComponent *install,
										GuyumInstallComponentClass *class);
														 
static void guyum_install_component_set_property(	GObject *object,
												guint property_id,
												const GValue *value,
												GParamSpec *pspec);
													
static void guyum_install_component_get_property(	GObject *object,
												guint property_id,
												GValue *value,
												GParamSpec *pspec);


/**
* Our "private" properties
*/
enum {
	INSTALL_REPOSITORIES=1,
	INSTALL_DEPS,
    INSTALL_REPOID,
};

/****************************************************************************
* typical functions															
****************************************************************************/
/**
* register new type 
*/
GType guyum_install_component_get_type(void) {
	static GType object_type = 0;
	if (!object_type) {
		static const GTypeInfo object_info = {
			sizeof(GuyumInstallComponentClass),
			NULL,
			NULL,
			(GClassInitFunc)guyum_install_component_class_init,
			NULL,
			NULL,
			sizeof(GuyumInstallComponent),
			0,
			(GInstanceInitFunc)guyum_install_component_init,
			NULL,
		};
		object_type = g_type_register_static(G_TYPE_OBJECT, "GuyumInstallComponent", &object_info, 0);		
	}
	return object_type;
}


/**
* init class
*/
static void guyum_install_component_class_init(GuyumInstallComponentClass *class) {
	GObjectClass *gobject_class = G_OBJECT_CLASS(class);
	GParamSpec *param_spec;

	/* Install features 
	* http://developer.gnome.org/doc/API/2.0/gobject/gobject-properties.html
	* create new objects:
	* 		g_object_new(G_OBJECT(...), "property1", "value1", "property2", value2, ... NULL)
	*/
	gobject_class->set_property = guyum_install_component_set_property;
	gobject_class->get_property = guyum_install_component_get_property;

	/* repositories */
	param_spec=g_param_spec_pointer("reps", 
									"repositories", 
									"guyum installation repositories", 
									G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class, 
									INSTALL_REPOSITORIES, 
									param_spec);
	
	/* deps */
	param_spec=g_param_spec_pointer("deps", 
									"dep list", 
									"guyum package dependencies", 
									G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class, 
									INSTALL_DEPS, 
									param_spec);

	/* repoid */
	param_spec = g_param_spec_string("repoid", 
									"install repoid", 
									"repoid", 
									"", 
									G_PARAM_READWRITE);	
	g_object_class_install_property(gobject_class, 
									INSTALL_REPOID, 
									param_spec);

}

/**
* init instance
*/
static void guyum_install_component_init(GuyumInstallComponent *install,
										 GuyumInstallComponentClass *class) 
{												 
	GuyumInstallComponent *self = (GuyumInstallComponent*)install;
	self->priv = g_new0(GuyumInstallComponentPrivate, 1);

	self->priv->deps = g_object_new(GUYUM_TYPE_PACKAGE_COMPOSITE, NULL);
	
	self->priv->guyuminstall_module = PyImport_ImportModule("guyuminstall");
	self->priv->guyuminstall_dict = PyModule_GetDict(self->priv->guyuminstall_module);
	self->priv->guyuminstall_class = PyDict_GetItemString(self->priv->guyuminstall_dict, "GuyumInstall");
	
	if (PyCallable_Check(self->priv->guyuminstall_class)) 
	{
		self->priv->guyuminstall_instance = PyObject_CallObject(self->priv->guyuminstall_class, NULL);
	}
	
	/* init object */
	PyObject* py_init = PyObject_CallMethod(self->priv->guyuminstall_instance, "init", NULL);
	Py_DECREF(py_init);	
	
}


/**
* set properties
*/
static void guyum_install_component_set_property(	GObject *object,
												guint property_id,
												const GValue *value,
												GParamSpec *pspec) {	
    GuyumInstallComponent *self = (GuyumInstallComponent*)object;									
    switch(property_id) {
        case INSTALL_REPOID: {
            if (self->priv->repoid!=NULL) {
                g_free(self->priv->repoid);
            }
            self->priv->repoid = g_value_dup_string(value);
        }
        break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID(object,property_id,pspec);
    		break;
    }		
}

/**
* get properties
*/
static void guyum_install_component_get_property(	GObject *object,
													guint property_id,
													GValue *value,
													GParamSpec *pspec) {	
	GuyumInstallComponent *self = (GuyumInstallComponent*)object;
														
	switch(property_id) {
		case INSTALL_DEPS: {
			g_value_set_pointer(value, self->priv->deps);
		}
		break;
		case INSTALL_REPOID: {
			g_value_set_string(value, self->priv->repoid);			
		}
		break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object,property_id,pspec);
    		break;
  	}	
}

/**
*
* destructor
*/
void guyum_install_component_destroy(GuyumInstallComponent *install) {
	g_return_if_fail(install!=NULL);
	g_return_if_fail(GUYUM_IS_INSTALL_COMPONENT(install));
	if (install->priv!=NULL) {
		if (install->priv->deps!=NULL) {
			guyum_package_composite_destroy(install->priv->deps);
		}
		/* clean up python stuff */
		if (install->priv->guyuminstall_deps!=NULL && PyCallable_Check(install->priv->guyuminstall_deps)) {
			Py_DECREF(install->priv->guyuminstall_deps);
		}
        if (install->priv->repoid!=NULL) {
            g_free(install->priv->repoid);
        }

		Py_DECREF(install->priv->guyuminstall_module);
		Py_DECREF(install->priv->guyuminstall_instance);		
		g_free(install->priv);
	}
}

/****************************************************************************
* install component functions												
****************************************************************************/

/**
*
*/
void guyum_install_component_install(GuyumInstallComponent* self)
{
	g_return_if_fail(self!=NULL);
	g_return_if_fail(GUYUM_IS_INSTALL_COMPONENT(self));

	PyObject* retobject = PyObject_CallMethod( self->priv->guyuminstall_instance, "installpackage", NULL);	
	if (retobject!=NULL) {
		Py_DECREF(retobject);
	}
}

/**
*
*/
guint guyum_install_component_preinstall(GuyumInstallComponent* self, PyObject* py_package, GuyumPackageComponent* package)
{

	int i = 0;
	guint retint = 0;
	PyObject* rescode = NULL;
	g_return_val_if_fail(self!=NULL, 100);
	g_return_val_if_fail(GUYUM_IS_INSTALL_COMPONENT(self), 100);
	
	self->priv->deps = g_object_new(GUYUM_TYPE_PACKAGE_COMPOSITE, NULL);
	
	rescode = PyObject_CallMethod(self->priv->guyuminstall_instance, "resolvedependencies", "(O)", py_package);
	if (PyInt_Check(rescode)) {

		if (PyInt_AsLong(rescode)==2) {
			gchar* repoid;
			retint = PyInt_AsLong(rescode);
			self->priv->guyuminstall_deps = PyObject_CallMethod(self->priv->guyuminstall_instance, "prepareinstall", NULL);
			if (self->priv->guyuminstall_deps!=NULL && PyList_Check(self->priv->guyuminstall_deps)) {
				/* Iterate packages. Packages are dictionaries in a list */
				for (i=0; i<PyList_Size(self->priv->guyuminstall_deps); i++) {
					GuyumPackageComponent* guyum_package;
					gboolean added;
					
					PyObject *py_package = PyList_GetItem(self->priv->guyuminstall_deps, i);
			
					guyum_package = g_object_new(GUYUM_TYPE_PACKAGE_COMPONENT, 
												"arch", PyString_AsString(PyObject_GetAttrString(py_package, "arch")),
												"changelog", PyString_AsString(PyObject_GetAttrString(py_package, "changelog")),
												"compsize", PyString_AsString(PyObject_GetAttrString(py_package, "compsize")),
												"conflicts", Py_List_As_GList(PyObject_GetAttrString(py_package, "conflicts")),
												"description", PyString_AsString(PyObject_GetAttrString(py_package, "description")),
												"epoch", PyString_AsString(PyObject_GetAttrString(py_package, "epoch")),
												"files", Py_List_As_GList(PyObject_GetAttrString(py_package, "files")),
												"group", PyString_AsString(PyObject_GetAttrString(py_package, "group")),
												"licenses", PyString_AsString(PyObject_GetAttrString(py_package, "licenses")),
												"localrelease", PyString_AsString(PyObject_GetAttrString(py_package, "localrelease")),
												"localversion", PyString_AsString(PyObject_GetAttrString(py_package, "localversion")),
												"name", PyString_AsString(PyObject_GetAttrString(py_package, "name")),
												"obsoletes", Py_List_As_GList(PyObject_GetAttrString(py_package, "obsoletes")),
												"packageid",PyString_AsString(PyObject_GetAttrString(py_package, "packageid")),
												"pkgsize", PyString_AsString(PyObject_GetAttrString(py_package, "compsize")),
												"provides", Py_List_As_GList(PyObject_GetAttrString(py_package, "provides")),
												"release", PyString_AsString(PyObject_GetAttrString(py_package, "release")),
												"remoterelease", PyString_AsString(PyObject_GetAttrString(py_package, "remoterelease")),			
												"remoteversion", PyString_AsString(PyObject_GetAttrString(py_package, "remoteversion")),
												"repoid", PyString_AsString(PyObject_GetAttrString(py_package, "repoid")),
												"requires", Py_List_As_GList(PyObject_GetAttrString(py_package, "requires")),
												"state", PyString_AsString(PyObject_GetAttrString(py_package, "state")),
												"summary", PyString_AsString(PyObject_GetAttrString(py_package, "summary")),
												"url", PyString_AsString(PyObject_GetAttrString(py_package, "url")),
												"version", PyString_AsString(PyObject_GetAttrString(py_package, "version")),
												"vendor", PyString_AsString(PyObject_GetAttrString(py_package, "vendor")),
												//"yumpackage", py_package,
												NULL
												);
					guyum_package_composite_add_component(self->priv->deps, guyum_package, &added);
					if (!added) {
						guyum_package_component_destroy(guyum_package);
					} 					
				}
			}
            GOBJECT_GET_STRING_PROPERTY(package, "repoid", &repoid);
            GOBJECT_SET_STRING_PROPERTY(self, "repoid", repoid);
            g_free(repoid);
            
		} else {
			retint = PyInt_AsLong(rescode);
		}
	} else {
		retint = 100;
	}
	return retint;
}

/**
*
*/
gchar* 
guyum_install_component_get_totalamount(GuyumInstallComponent* self)
{
	PyObject* retamount;
	gchar* retvalue = NULL;
	g_return_val_if_fail(self!=NULL, NULL);
	g_return_val_if_fail(GUYUM_IS_INSTALL_COMPONENT(self), NULL);
	
	retamount = PyObject_CallMethod(self->priv->guyuminstall_instance, "gettotalamount", NULL);
	retvalue = PyString_AsString(retamount);
	Py_DECREF(retamount);
	
	return retvalue;
}
