/***************************************************************************
 *            guyum_main.c
 *
 *  $LastChangedDate: 2005-12-26 17:59:28 +0100 (Mo, 26 Dez 2005) $
 *  Copyright   2005  $Author: vklein $
 *  Email
 ****************************************************************************/

#include <include/guyum/gy_main.h>


/****************************************************************************
* Function declarations													
****************************************************************************/
static void guyum_main_class_init			(GuyumMainClass *class);

static void guyum_main_init					(GuyumMain *main,
												 GuyumMainClass *class);
														 
static void guyum_main_set_property(	GObject *object,
										guint property_id,
										const GValue *value,
										GParamSpec *pspec);
													
static void guyum_main_get_property(	GObject *object,
										guint property_id,
										GValue *value,
										GParamSpec *pspec);


/**
* Our "private" properties
*/
enum {
	MAIN_QUERIES=1,
	MAIN_REPOS,
	MAIN_LOGFILENAME,
	MAIN_THRESHOLD,	
	MAIN_INSTALL,
};

/****************************************************************************
* typical functions															
****************************************************************************/
/**
* register new type 
*/
GType guyum_main_get_type(void) {
	static GType object_type = 0;
	if (!object_type) {
		static const GTypeInfo object_info = {
			sizeof(GuyumMainClass),
			NULL,
			NULL,
			(GClassInitFunc)guyum_main_class_init,
			NULL,
			NULL,
			sizeof(GuyumMain),
			0,
			(GInstanceInitFunc)guyum_main_init,
			NULL,
		};
		object_type = g_type_register_static(G_TYPE_OBJECT, "GuyumMain", &object_info, 0);		
	}
	return object_type;
}


/**
* class initializer
*/
static void guyum_main_class_init(GuyumMainClass *class) {
	GObjectClass *gobject_class = G_OBJECT_CLASS(class);
	GParamSpec *param_spec;	
	
	/* Install features 
	* http://developer.gnome.org/doc/API/2.0/gobject/gobject-properties.html
	* create new objects:
	* 		g_object_new(G_OBJECT(...), "property1", "value1", "property2", value2, ... NULL)
	*/
	gobject_class->set_property = guyum_main_set_property;
	gobject_class->get_property = guyum_main_get_property;	
	
	/* queries */
	param_spec=g_param_spec_pointer("queries", 
									"querylist", 
									"guyum queries", 
									G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class, 
									MAIN_QUERIES, 
									param_spec);	
	
	/* repos */
	param_spec=g_param_spec_pointer("repos", 
									"repositories", 
									"yum repositories", 
									G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class, 
									MAIN_REPOS, 
									param_spec);
	/* logfilename */
	param_spec = g_param_spec_string("logfilename", 
									"logfile name", 
									"logfile", 
									"", 
									G_PARAM_READWRITE);	
	g_object_class_install_property(gobject_class, 
									MAIN_LOGFILENAME, 
									param_spec);

	/* threshold */	
	param_spec = g_param_spec_int(	"threshold", 
									"thres", 
									"", 
									0, 
									10, 
									5, 
									G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class, 
									MAIN_THRESHOLD, 
									param_spec);									

	/* install component */
	param_spec=g_param_spec_pointer("install", 
									"installcomponent", 
									"guyum install", 
									G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class, 
									MAIN_INSTALL, 
									param_spec);	

}

/**
* instance initializer
*/
static void guyum_main_init(	GuyumMain *main,
								GuyumMainClass *class) 
{	
	PyObject* retinit;
	PyObject* retreposetup ;
	GuyumMain *self = (GuyumMain*)main;
	int i = 0;								
	self->priv = g_new0(GuyumMainPrivate, 1);
	self->priv->queries = g_object_new(GUYUM_TYPE_QUERY_COMPOSITE, NULL);
	self->priv->repos = g_object_new(GUYUM_TYPE_REPO_COMPOSITE, NULL);
	
	/* Init python modules */
	self->priv->yumquery_function_init = g_strdup("init");
	
	Py_Initialize();
	
	self->priv->yumquery_module = PyImport_ImportModule("guyumquery");	
	self->priv->yumquery_dict = PyModule_GetDict(self->priv->yumquery_module);
	self->priv->yumquery_class = PyDict_GetItemString(self->priv->yumquery_dict, "GuyumQuery");
	
	if (PyCallable_Check(self->priv->yumquery_class)) 
	{
		self->priv->yumquery_instance = PyObject_CallObject(self->priv->yumquery_class, NULL);
	} 
	
	retinit = PyObject_CallMethod(self->priv->yumquery_instance, 
											self->priv->yumquery_function_init, 
											NULL);
	
	retreposetup = PyObject_CallMethod(self->priv->yumquery_instance, "doRepoSetup", NULL);
	
	/* Test for errors */
	if (retreposetup==NULL) {		
		GUYUM_PYTHON_FATAL_ERROR();
		exit(1);
	} else {
	
		/* create repository view */
		self->priv->yumquery_repos = PyObject_CallMethod(self->priv->yumquery_instance, "getrepos", NULL);
		
		/* Iterate repositories. Repositories are dictionaries in a list */
		for (i=0; i<PyList_Size(self->priv->yumquery_repos); i++) {
			GuyumRepoComponent* guyum_repo;
			PyObject *py_repo_id;
			PyObject *py_repo_name;
			PyObject *py_repo_enabled;			
			PyObject *py_repo;

			py_repo	= PyList_GetItem(self->priv->yumquery_repos, i);
	
			guyum_repo = g_object_new(GUYUM_TYPE_REPO_COMPONENT, NULL);
			
			py_repo_id = PyDict_GetItemString(py_repo, "id");
			py_repo_name = PyDict_GetItemString(py_repo, "name");
			py_repo_enabled = PyDict_GetItemString(py_repo, "enabled");
			
			GOBJECT_SET_STRING_PROPERTY(guyum_repo, "id", PyString_AsString(py_repo_id));
			GOBJECT_SET_STRING_PROPERTY(guyum_repo, "name", PyString_AsString(py_repo_name));
	
			if (g_strcasecmp(PyString_AsString(py_repo_enabled), "0")==0)  {
				GOBJECT_UNSET_BOOLEAN_PROPERTY(guyum_repo, "enabled");				
			} else {				
				GOBJECT_SET_BOOLEAN_PROPERTY(guyum_repo, "enabled");				
			}
			guyum_repo_composite_add_component(self->priv->repos, guyum_repo);
			
		}
		Py_DECREF(self->priv->yumquery_repos);
	}
	Py_DECREF(retreposetup);
	Py_DECREF(retinit);
}

/**
*
* Destructors 
*/
void guyum_main_destroy(GuyumMain *main) {
	g_return_if_fail(main!=NULL);
	g_return_if_fail(GUYUM_IS_MAIN(main));
	if (main->priv!=NULL) {		
		if (main->priv->queries!=NULL) {
			guyum_query_composite_destroy(main->priv->queries);
		}
		if (main->priv->repos!=NULL) {
			guyum_repo_composite_destroy(main->priv->repos);
		}		
		if (main->priv->logfilename!=NULL) {
			g_free(main->priv->logfilename);
		}
		if (main->priv->install!=NULL) {
			guyum_install_component_destroy(main->priv->install);
		}			
		Py_DECREF(main->priv->yumquery_instance);
		Py_DECREF(main->priv->yumquery_module);
		g_free(main->priv->yumquery_function_init);
		
		g_free(main->priv);				
		
		Py_Finalize();
	}
}


/**
* set properties
*/
static void guyum_main_set_property(	GObject *object,
										guint property_id,
										const GValue *value,
										GParamSpec *pspec) {
	GuyumMain *self = (GuyumMain*)object;
										
	switch(property_id) {
		case MAIN_QUERIES: {
			if (self->priv->queries!=NULL) {
				guyum_query_composite_destroy(self->priv->queries);			
				self->priv->queries = NULL;
			}
			self->priv->queries = g_value_get_pointer(value);			
		}
		break;
		case MAIN_REPOS: {
			if (self->priv->repos!=NULL) {
				guyum_repo_composite_destroy(self->priv->repos);			
				self->priv->repos = NULL;
			}
			self->priv->repos = g_value_get_pointer(value);			
		}
		break;
		case MAIN_THRESHOLD: {
			self->priv->threshold = g_value_get_int(value);			
		}
		break;		
		case MAIN_LOGFILENAME: {
			if (self->priv->logfilename!=NULL) {
				g_free(self->priv->logfilename);
			}
			self->priv->logfilename = g_value_dup_string(value);			
		}
		break;	
		case MAIN_INSTALL: {
			if (self->priv->install!=NULL) {
				guyum_install_component_destroy(self->priv->install);
			}
			self->priv->install = g_value_get_pointer(value);			
		}
		break;	

		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object,property_id,pspec);
    		break;
  	}
		
}

/**
* get properties
*/
static void guyum_main_get_property(	GObject *object,
										guint property_id,
										GValue *value,
										GParamSpec *pspec) {	
	GuyumMain *self = (GuyumMain*)object;
														
	switch(property_id) {
		case MAIN_QUERIES: {
			g_value_set_pointer(value, self->priv->queries);
		}
		break;
		case MAIN_REPOS: {
			g_value_set_pointer(value, self->priv->repos);
		}
		break;	
		case MAIN_LOGFILENAME: {
			g_value_set_string(value, self->priv->logfilename);			
		}
		break;		
		case MAIN_THRESHOLD: {
			g_value_set_int(value, self->priv->threshold);
		}
		break;		
		case MAIN_INSTALL: {
			g_value_set_pointer(value, self->priv->install);
		}
		break;	

		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object,property_id,pspec);
    		break;
  	}	
}

/**
*
*/
gboolean guyum_main_add_query(GuyumMain *self, const gchar* query)
{
	g_return_val_if_fail(self!=NULL, FALSE);
	guyum_query_composite_add_component(self->priv->queries, 
										(GuyumQueryComponent*) g_object_new(GUYUM_TYPE_QUERY_COMPONENT, 
																			"searchstring", query, 
																			NULL));
	return TRUE;
}

/**
*
*/
GuyumQueryComponent* guyum_main_get_nth_query(GuyumMain *self,
												gint position)
{
	g_return_val_if_fail(self!=NULL, NULL);	
	GuyumQueryComponent* query = guyum_query_composite_get_nth_component(self->priv->queries, position);
	return query;
}



/**
*
*/
gint guyum_main_get_query_position(GuyumMain *self,
									GuyumQueryComponent* query)
{
	g_return_val_if_fail(self!=NULL, -1);	
	g_return_val_if_fail(GUYUM_IS_MAIN(self), -1);
	g_return_val_if_fail(query!=NULL, -1);	
	g_return_val_if_fail(GUYUM_IS_QUERY_COMPONENT(query), -1);
	gint i = 0;
	gint retpageindex = -1;
	for( i=0; i<guyum_query_composite_get_component_count(self->priv->queries) && retpageindex==-1; i++) {
		if (query==guyum_query_composite_get_nth_component(self->priv->queries, i)) {
			retpageindex = i;
		}
	}
	return retpageindex;
}

/**
*
*/
void 
guyum_main_remove_nth_query(GuyumMain *self, 
							GuyumQueryComponent* query)
{
	g_return_if_fail(self!=NULL);	
	guyum_query_composite_remove_component(self->priv->queries, query);
}

/**
*
*/
GuyumPackageComposite* guyum_main_get_nth_packages(GuyumMain *self,
												gint position)
{
	g_return_val_if_fail(self!=NULL, NULL);	
	GuyumPackageComposite* retpackages = NULL;
	GuyumQueryComponent* query = guyum_query_composite_get_nth_component(self->priv->queries, position);
	GOBJECT_GET_POINTER_PROPERTY(query, "packages", retpackages);
	return retpackages;
}

/**
*
*/
GuyumQueryComponent* guyum_main_init_query(GuyumMain *self,
											gint position)
{
	g_return_val_if_fail(self!=NULL, NULL);	
	GuyumQueryComponent* newquery = g_object_new(GUYUM_TYPE_QUERY_COMPONENT,
												NULL);

	guyum_query_setlog(newquery, self->priv->logfilename, self->priv->threshold);	
	guyum_query_composite_insert_component(self->priv->queries, newquery, position);
	
	return newquery;
}
											

/**
*
*/
GuyumPackageComposite* guyum_main_execute_query(GuyumMain *self, 
												GuyumQueryComponent* newquery,
												const gchar* query,
												GList* searchlist, 
												gchar* repoid)
{
	g_return_val_if_fail(self!=NULL, NULL);
	g_return_val_if_fail(newquery!=NULL, NULL);
	g_return_val_if_fail(query!=NULL, NULL);
	g_return_val_if_fail(searchlist!=NULL, NULL);
	g_return_val_if_fail(repoid!=NULL, NULL);
	
	GuyumPackageComposite* packages = NULL;
	
	if (guyum_query_component_search(newquery, query, searchlist, repoid)==GY_SUCCESS) {
		GOBJECT_GET_POINTER_PROPERTY(newquery, "packages", packages);
	} else {
		return NULL;
	}

	return packages;
}

/**
* After installation, rerun search
*/
GuyumPackageComposite* guyum_main_rpmdbsearch(GuyumMain* self,
											    int query_component_nth,
                                                gchar* repoid)
{
	g_return_val_if_fail(self!=NULL, NULL);
	GuyumPackageComposite* packages;
	GuyumQueryComponent* query = guyum_query_composite_get_nth_component(self->priv->queries, 
																		query_component_nth);	
	
	guyum_query_component_rpmdbsearch(query, repoid);	
	GOBJECT_GET_POINTER_PROPERTY(query, "packages", packages);
	
	return packages;
}

/**
*
*/
GuyumPackageComposite* guyum_main_finddeps(GuyumMain *self, 
											int query_component_nth, 											
											const gchar* packageid)
{
	g_return_val_if_fail(self!=NULL, NULL);
	GuyumPackageComposite* packages = NULL;
    GuyumPackageComposite* querypackages = NULL;
    GuyumPackageComponent* package = NULL;
	PyObject* py_package=NULL;
	GuyumInstallComponent* install=NULL;
	guint rescode = 0;
    gchar* package_repoid = NULL;
	
	GuyumQueryComponent* query = guyum_query_composite_get_nth_component(self->priv->queries, 
																		query_component_nth);
	py_package = guyum_query_component_get_package(query, packageid);
    GOBJECT_GET_POINTER_PROPERTY(G_OBJECT(query), "packages", querypackages);
    package = guyum_package_composite_get_id_component(querypackages, packageid);
	if (py_package!=NULL) {
		install = g_object_new(GUYUM_TYPE_INSTALL_COMPONENT, NULL);
		rescode = guyum_install_component_preinstall(install, py_package, package);
        GOBJECT_GET_STRING_PROPERTY(package, "repoid", &package_repoid);
		if (rescode==2) {
			GOBJECT_SET_POINTER_PROPERTY(self, "install", install);
			GOBJECT_GET_POINTER_PROPERTY(G_OBJECT(install), "deps", packages);
            GOBJECT_SET_STRING_PROPERTY(install, "repoid", package_repoid);
			Py_DECREF(py_package);
            g_free(package_repoid);
		} else {
			
		}
	}
	return packages;
}

/**
*
*/
gchar* 
guyum_main_get_totalamount(GuyumMain *self, int query_component_nth)
{
	gchar* retvalue;
	g_return_val_if_fail(self!=NULL, NULL);
	retvalue = guyum_install_component_get_totalamount(self->priv->install);
	return retvalue;
}

/**
*
*/
void guyum_main_execute_install(GuyumMain* self, int query_component_nth)
{
	g_return_if_fail(self!=NULL);
	guyum_install_component_install(self->priv->install);
}


/**
*
*/
gchar* guyum_main_get_repo_property(GuyumMain* self, const gchar* repoid, const gchar* property)
{
	PyObject *value = PyObject_CallMethod(self->priv->yumquery_instance, "getrepovalue", "(ss)", repoid, property);
	gchar* retvalue = NULL;
	if (value!=NULL) {
		retvalue = PyString_AsString(value);
		Py_DECREF(value);
	}
	return retvalue;
}

void
guyum_main_set_renew(GuyumMain* self, int current)
{
	g_return_if_fail(self!=NULL);
	g_return_if_fail(current>-1);
	guyum_query_composite_set_renew(self->priv->queries, current);
}

gboolean
guyum_main_get_renew(GuyumMain* self, int current)
{
	g_return_val_if_fail(self!=NULL, FALSE);
	g_return_val_if_fail(current>-1, FALSE);
	return guyum_query_composite_get_renew(self->priv->queries, current);
}
