/***************************************************************************
 *            guyum_package_component.c
 *
 *  $LastChangedDate: 2005-12-26 17:59:28 +0100 (Mo, 26 Dez 2005) $
 *  Copyright   2005  $Author: vklein $
 *  Email
 ****************************************************************************/

#include <include/guyum/gy_package.h>


/****************************************************************************
* Function declarations													
****************************************************************************/
static void guyum_package_component_class_init			(GuyumPackageComponentClass *class);

static void guyum_package_component_init			(GuyumPackageComponent *package,
												 	GuyumPackageComponentClass *class);
														 
static void guyum_package_component_set_property(	GObject *object,
													guint property_id,
													const GValue *value,
													GParamSpec *pspec);
													
static void guyum_package_component_get_property(	GObject *object,
													guint property_id,
													GValue *value,
													GParamSpec *pspec);


/**
* Our "private" properties
*/
enum {	
	PACKAGE_ARCH=1,
	PACKAGE_CHANGELOG,
	PACKAGE_COMPSIZE,
	PACKAGE_CONFLICTS,
	PACKAGE_DESCRIPTION,
	PACKAGE_EPOCH,
	PACKAGE_FILES,
	PACKAGE_GROUP,
	PACKAGE_LICENSES,
	PACKAGE_LOCALRELEASE,
	PACKAGE_LOCALVERSION,
	PACKAGE_NAME,	
	PACKAGE_OBSOLETES,	
	PACKAGE_PACKAGEID,
	PACKAGE_PKGSIZE,
	PACKAGE_PROVIDES,
	PACKAGE_RELEASE,
	PACKAGE_REMOTERELEASE,
	PACKAGE_REMOTEVERSION,	
	PACKAGE_REPOID,
	PACKAGE_REQUIRES,
	PACKAGE_STATE,
	PACKAGE_SUMMARY,
	PACKAGE_URL,
	PACKAGE_VENDOR,
	PACKAGE_VERSION,	
	PACKAGE_YUMPACKAGE,
};

/****************************************************************************
* typical functions															
****************************************************************************/
/**
* register new type 
*/
GType guyum_package_component_get_type(void) {
	static GType object_type = 0;
	if (!object_type) {
		static const GTypeInfo object_info = {
			sizeof(GuyumPackageComponentClass),
			NULL,
			NULL,
			(GClassInitFunc)guyum_package_component_class_init,
			NULL,
			NULL,
			sizeof(GuyumPackageComponent),
			0,
			(GInstanceInitFunc)guyum_package_component_init,
			NULL,
		};
		object_type = g_type_register_static(G_TYPE_OBJECT, "GuyumPackageComponent", &object_info, 0);		
	}
	return object_type;
}


/**
* class initializer
*/
static void guyum_package_component_class_init(GuyumPackageComponentClass *class) {
	GObjectClass *gobject_class = G_OBJECT_CLASS(class);
	
	/* Install features 
	* http://developer.gnome.org/doc/API/2.0/gobject/gobject-properties.html
	* create new objects:
	* 		g_object_new(G_OBJECT(...), "property1", "value1", "property2", value2, ... NULL)
	*/
	gobject_class->set_property = guyum_package_component_set_property;
	gobject_class->get_property = guyum_package_component_get_property;	
	
	GOBJECT_INSTALL_STRING_PROPERTY(gobject_class, "arch", PACKAGE_ARCH);
	GOBJECT_INSTALL_STRING_PROPERTY(gobject_class, "changelog", PACKAGE_CHANGELOG);
	GOBJECT_INSTALL_STRING_PROPERTY(gobject_class, "compsize", PACKAGE_COMPSIZE);
	GOBJECT_INSTALL_POINTER_PROPERTY(gobject_class, "conflicts", PACKAGE_CONFLICTS);
	GOBJECT_INSTALL_STRING_PROPERTY(gobject_class, "description", PACKAGE_DESCRIPTION);
	GOBJECT_INSTALL_STRING_PROPERTY(gobject_class, "epoch", PACKAGE_EPOCH);
	GOBJECT_INSTALL_POINTER_PROPERTY(gobject_class, "files", PACKAGE_FILES);
	GOBJECT_INSTALL_STRING_PROPERTY(gobject_class, "group", PACKAGE_GROUP);
	GOBJECT_INSTALL_STRING_PROPERTY(gobject_class, "licenses", PACKAGE_LICENSES);
	GOBJECT_INSTALL_STRING_PROPERTY(gobject_class, "localrelease", PACKAGE_LOCALRELEASE);
	GOBJECT_INSTALL_STRING_PROPERTY(gobject_class, "localversion", PACKAGE_LOCALVERSION);	
	GOBJECT_INSTALL_STRING_PROPERTY(gobject_class, "name", PACKAGE_NAME);
	GOBJECT_INSTALL_POINTER_PROPERTY(gobject_class, "obsoletes", PACKAGE_OBSOLETES);
	GOBJECT_INSTALL_STRING_PROPERTY(gobject_class, "packageid", PACKAGE_PACKAGEID);
	GOBJECT_INSTALL_STRING_PROPERTY(gobject_class, "pkgsize", PACKAGE_PKGSIZE);
	GOBJECT_INSTALL_POINTER_PROPERTY(gobject_class, "provides", PACKAGE_PROVIDES);
	GOBJECT_INSTALL_STRING_PROPERTY(gobject_class, "release", PACKAGE_RELEASE);
	GOBJECT_INSTALL_STRING_PROPERTY(gobject_class, "remoterelease", PACKAGE_REMOTERELEASE);
	GOBJECT_INSTALL_STRING_PROPERTY(gobject_class, "remoteversion", PACKAGE_REMOTEVERSION);	
	GOBJECT_INSTALL_STRING_PROPERTY(gobject_class, "repoid", PACKAGE_REPOID);	
	GOBJECT_INSTALL_POINTER_PROPERTY(gobject_class, "requires", PACKAGE_REQUIRES);
	GOBJECT_INSTALL_STRING_PROPERTY(gobject_class, "state", PACKAGE_STATE);	
	GOBJECT_INSTALL_STRING_PROPERTY(gobject_class, "summary", PACKAGE_SUMMARY);	
	GOBJECT_INSTALL_STRING_PROPERTY(gobject_class, "url", PACKAGE_URL);	
	GOBJECT_INSTALL_STRING_PROPERTY(gobject_class, "vendor", PACKAGE_VENDOR);	
	GOBJECT_INSTALL_STRING_PROPERTY(gobject_class, "version", PACKAGE_VERSION);	
	GOBJECT_INSTALL_POINTER_PROPERTY(gobject_class, "yumpackage", PACKAGE_YUMPACKAGE);

}

/**
* instance initializer
*/
static void guyum_package_component_init(	GuyumPackageComponent *package,
											GuyumPackageComponentClass *class) {	
											 
	GuyumPackageComponent *self = (GuyumPackageComponent*)package;
	self->priv = g_new0(GuyumPackageComponentPrivate, 1);

}

/**
*
* Destructors 
*/
void guyum_package_component_destroy(GuyumPackageComponent *package) {
	g_return_if_fail(package!=NULL);
	g_return_if_fail(GUYUM_IS_PACKAGE_COMPONENT(package));
	
	if (package->priv!=NULL) {
		g_free(package->priv->arch);
		g_free(package->priv->changelog);
		g_free(package->priv->compsize);
		g_list_free(package->priv->conflicts);		
		g_free(package->priv->description);
		g_free(package->priv->epoch);
		g_list_free(package->priv->files);
		g_free(package->priv->group);
		g_free(package->priv->licenses);				
		g_free(package->priv->localrelease);
		g_free(package->priv->localversion);		
		g_free(package->priv->name);
		g_list_free(package->priv->obsoletes);		
		g_free(package->priv->packageid);
		g_free(package->priv->pkgsize);
		g_list_free(package->priv->provides);
		g_free(package->priv->release);		
		g_free(package->priv->remoterelease);
		g_free(package->priv->remoteversion);		
		g_free(package->priv->repoid);
		g_list_free(package->priv->requires);
		g_free(package->priv->state);
		g_free(package->priv->summary);
		g_free(package->priv->url);
		g_free(package->priv->vendor);
		g_free(package->priv->version);
		
		
		if (package->priv->yumpackage!=NULL) {			
			PyObject_Del(package->priv->yumpackage);
		}
		g_free(package->priv);
	}
	
}


/**
* set properties
*/
static void guyum_package_component_set_property(	GObject *object,
													guint property_id,
													const GValue *value,
													GParamSpec *pspec) {
	GuyumPackageComponent *self = (GuyumPackageComponent*)object;
										
	switch(property_id) {
		case PACKAGE_ARCH: {
			g_free(self->priv->name);
			self->priv->arch = g_value_dup_string(value);			
		}
		break;
		case PACKAGE_CHANGELOG: {
			g_free(self->priv->changelog);
			self->priv->changelog= g_value_dup_string(value);			
		}
		break;
		case PACKAGE_COMPSIZE: {
			g_free(self->priv->compsize);
			self->priv->compsize = g_value_dup_string(value);			
		}
		break;
		case PACKAGE_CONFLICTS: {
			g_list_free(self->priv->conflicts);
			self->priv->conflicts = g_value_get_pointer(value);			
		}
		break;
		case PACKAGE_DESCRIPTION: {
			g_free(self->priv->description);
			self->priv->description = g_value_dup_string(value);
		}
		break;
		case PACKAGE_EPOCH: {
			g_free(self->priv->epoch);
			self->priv->epoch = g_value_dup_string(value);
		}
		break;
		case PACKAGE_FILES: {
			g_list_free(self->priv->files);
			self->priv->files = g_value_get_pointer(value);
		}
		break;

		case PACKAGE_GROUP: {
			g_free(self->priv->group);
			self->priv->group = g_value_dup_string(value);
		}
		break;
		case PACKAGE_LICENSES: {
			g_free(self->priv->licenses);
			self->priv->licenses = g_value_dup_string(value);
		}
		break;
		case PACKAGE_LOCALRELEASE: {
			g_free(self->priv->localrelease);
			self->priv->localrelease = g_value_dup_string(value);
		}
		break;		
		case PACKAGE_LOCALVERSION: {
			g_free(self->priv->localversion);
			self->priv->localversion = g_value_dup_string(value);
		}
		break;	
		case PACKAGE_NAME: {
			g_free(self->priv->name);
			self->priv->name = g_value_dup_string(value);
		}
		break;
		case PACKAGE_OBSOLETES: {
			g_list_free(self->priv->obsoletes);
			self->priv->obsoletes = g_value_get_pointer(value);			
		}
		break;		
		case PACKAGE_PACKAGEID: {
			g_free(self->priv->packageid);
			self->priv->packageid = g_value_dup_string(value);
		}
		break;
		case PACKAGE_PKGSIZE: {
			g_free(self->priv->pkgsize);
			self->priv->pkgsize = g_value_dup_string(value);
		}
		break;	
		case PACKAGE_PROVIDES: {
			if (self->priv->provides!=NULL && g_list_length(self->priv->provides)) {
				g_list_free(self->priv->provides);
			}
			self->priv->provides = g_value_get_pointer(value);			
		}
		break;		
		case PACKAGE_RELEASE: {
			g_free(self->priv->release);
			self->priv->release = g_value_dup_string(value);
		}
		break;	
		case PACKAGE_REMOTERELEASE: {
			g_free(self->priv->remoterelease);
			self->priv->remoterelease = g_value_dup_string(value);
		}
		break;		
		case PACKAGE_REMOTEVERSION: {
			g_free(self->priv->remoteversion);
			self->priv->remoteversion = g_value_dup_string(value);
		}
		break;	
		case PACKAGE_REPOID: {
			g_free(self->priv->repoid);
			self->priv->repoid = g_value_dup_string(value);
		}
		break;		
		case PACKAGE_REQUIRES: {
			g_list_free(self->priv->requires);
			self->priv->requires = g_value_get_pointer(value);			
		}
		break;		
		case PACKAGE_STATE: {
			g_free(self->priv->state);
			self->priv->state = g_value_dup_string(value);
		}
		break;		
		case PACKAGE_SUMMARY: {
			g_free(self->priv->summary);
			self->priv->summary = g_value_dup_string(value);
		}
		break;		
		case PACKAGE_URL: {
			g_free(self->priv->url);
			self->priv->url = g_value_dup_string(value);
		}
		break;				
		case PACKAGE_VENDOR: {
			g_free(self->priv->vendor);
			self->priv->vendor = g_value_dup_string(value);
		}
		break;
		case PACKAGE_VERSION: {
			g_free(self->priv->version);
			self->priv->version = g_value_dup_string(value);
		}
		break;
		case PACKAGE_YUMPACKAGE: {
			self->priv->yumpackage = g_value_get_pointer(value);
		}
		break;

		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object,property_id,pspec);
    		break;
  	}
		
}

/**
* get properties
*/
static void guyum_package_component_get_property(	GObject *object,
													guint property_id,
													GValue *value,
													GParamSpec *pspec) {	
	GuyumPackageComponent *self = (GuyumPackageComponent*)object;
														
	switch(property_id) {
		case PACKAGE_ARCH: {
			g_value_set_string(value, self->priv->arch);
		}
		break;	
		case PACKAGE_CHANGELOG: {
			g_value_set_string(value, self->priv->changelog);
		}
		break;	
		case PACKAGE_COMPSIZE: {
			g_value_set_string(value, self->priv->compsize);
		}
		break;	
		case PACKAGE_CONFLICTS: {			
			g_value_set_pointer(value, self->priv->conflicts);
		}
		break;	
		case PACKAGE_DESCRIPTION: {
			g_value_set_string(value, self->priv->description);
		}
		break;
		case PACKAGE_EPOCH: {
			g_value_set_string(value, self->priv->epoch);
		}
		break;
		case PACKAGE_FILES: {
			g_value_set_pointer(value, self->priv->files);
		}
		break;	

		case PACKAGE_GROUP: {
			g_value_set_string(value, self->priv->group);
		}
		break;
		case PACKAGE_LICENSES: {
			g_value_set_string(value, self->priv->licenses);
		}
		break;
		case PACKAGE_LOCALRELEASE: {
			g_value_set_string(value, self->priv->localrelease);
		}
		break;		
		case PACKAGE_LOCALVERSION: {
			g_value_set_string(value, self->priv->localversion);
		}
		break;					
		case PACKAGE_NAME: {
			g_value_set_string(value, self->priv->name);
		}
		break;	
		case PACKAGE_OBSOLETES: {
			g_value_set_pointer(value, self->priv->obsoletes);
		}
		break;			
		case PACKAGE_PACKAGEID: {
			g_value_set_string(value, self->priv->packageid);
		}
		break;	
		case PACKAGE_PKGSIZE: {
			g_value_set_string(value, self->priv->pkgsize);
		}
		break;	
		case PACKAGE_PROVIDES: {
			g_value_set_pointer(value, self->priv->provides);
		}
		break;			
		case PACKAGE_RELEASE: {
			g_value_set_string(value, self->priv->release);
		}
		break;	
		case PACKAGE_REMOTERELEASE: {
			g_value_set_string(value, self->priv->remoterelease);
		}
		break;					
		case PACKAGE_REMOTEVERSION: {
			g_value_set_string(value, self->priv->remoteversion);
		}
		break;			
		case PACKAGE_REPOID: {
			g_value_set_string(value, self->priv->repoid);
		}
		break;
		case PACKAGE_REQUIRES: {			
			g_value_set_pointer(value, self->priv->requires);
		}
		break;			
		case PACKAGE_STATE: {
			g_value_set_string(value, self->priv->state);
		}
		break;	
		case PACKAGE_SUMMARY: {
			g_value_set_string(value, self->priv->summary);
		}
		break;	
		case PACKAGE_URL: {
			g_value_set_string(value, self->priv->url);
		}
		break;	
		case PACKAGE_VERSION: {
			g_value_set_string(value, self->priv->version);
		}
		break;	
		case PACKAGE_VENDOR: {
			g_value_set_string(value, self->priv->vendor);
		}
		break;	

		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object,property_id,pspec);
    		break;
  	}	
}
