/***************************************************************************
 *            guyum_package_composite.c
 *
 *  $LastChangedDate: 2005-12-31 11:25:00 +0100 (Sa, 31 Dez 2005) $
 *  Copyright  2005  $Author: vklein $
 *  Email
 ****************************************************************************/
#include <include/guyum/gy_package_list.h>



/****************************************************************************
* Function declarations													
****************************************************************************/


static void guyum_package_composite_init(GuyumPackageComposite *package,
										GuyumPackageCompositeClass *class);
														 


/****************************************************************************
* typical functions															
****************************************************************************/
/**
* register new type 
*/
GType guyum_package_composite_get_type(void) 
{
	static GType object_type = 0;
	if (!object_type) {
		static const GTypeInfo object_info = {
			sizeof(GuyumPackageCompositeClass),
			NULL,
			NULL,
			NULL,
			NULL,
			NULL,
			sizeof(GuyumPackageComposite),
			0,
			(GInstanceInitFunc)guyum_package_composite_init,
			NULL,
		};
		object_type = g_type_register_static(G_TYPE_OBJECT, "GuyumPackageComposite", &object_info, 0);		
	}
	return object_type;
}

/**
* init instance
*/
static void guyum_package_composite_init(GuyumPackageComposite *package,
										 GuyumPackageCompositeClass *class) 
{	
											 
	GuyumPackageComposite *self = (GuyumPackageComposite*)package;
	self->priv = g_new0(GuyumPackageCompositePrivate, 1);
	self->priv->package_components = NULL;
	self->priv->components_count = 0;
											 
}

/**
*
* destructor
*/
void guyum_package_composite_destroy(GuyumPackageComposite *package_composite) 
{
	g_return_if_fail(package_composite!=NULL);
	g_return_if_fail(GUYUM_IS_PACKAGE_COMPOSITE(package_composite));
	if (package_composite->priv!=NULL) {		
		if (package_composite->priv->package_components!=NULL) {
			gint i = 0;			
			for (; i<package_composite->priv->components_count; i++) {
				guyum_package_component_destroy(guyum_package_composite_get_nth_component(package_composite, i));				
			}
			g_list_free(package_composite->priv->package_components);
		}			
		g_free(package_composite->priv);
	}
}

/****************************************************************************/
/* composite functions														*/
/****************************************************************************/

/**
* add new package component to package composite
* @param package_composite "this" object
* @param package	GuyumPackageComponent
*/
void guyum_package_composite_add_component(	GuyumPackageComposite *self,
												GuyumPackageComponent *package,
											gboolean* added) 
{
	g_return_if_fail(self!=NULL);
	g_return_if_fail(GUYUM_IS_PACKAGE_COMPOSITE(self));
	g_return_if_fail(self->priv!=NULL);
	
	g_return_if_fail(package!=NULL);
	g_return_if_fail(GUYUM_IS_PACKAGE_COMPONENT(package));
	g_return_if_fail(package->priv!=NULL);	
	
	
	gboolean toadd = TRUE;
	
	if (g_strcasecmp(package->priv->repoid, "installed")==0) {
		gint i = 0;
		for (i=0; i<self->priv->components_count && toadd; i++) {
			GuyumPackageComponent* newpackage = g_list_nth_data(self->priv->package_components, i);
			if (g_strcasecmp(newpackage->priv->name, package->priv->name)==0) {
				toadd=FALSE;
			}		
		}
	}
	
	if (toadd) {
		self->priv->package_components = g_list_append(self->priv->package_components, 
																	(gpointer)package);
		self->priv->components_count++;	
	} 
		
	(*added)=toadd;
}


/**
* remove doubles
* @param package_composite "this" object
*/
void 
guyum_package_composite_remove_doubles(GuyumPackageComposite *self) 
{
	g_return_if_fail(self!=NULL);
	g_return_if_fail(self->priv!=NULL);
	gint i = 0;
	gint listcounter = 0;
	GList* toremove = NULL;
	
	for (i=0; i<self->priv->components_count; i++) {
		GuyumPackageComponent* package = g_list_nth_data(self->priv->package_components, i);
		gboolean notfound=TRUE;
		/* package on repos not found */
		if (g_strcasecmp(package->priv->repoid, "installed")==0) {
			gint j = 0;
			for (j=0; j<self->priv->components_count && notfound; j++) {
				if (i!=j) {
					GuyumPackageComponent* newpackage = g_list_nth_data(self->priv->package_components, j);
					if (g_strcasecmp(newpackage->priv->name, package->priv->name)==0) {
						toremove = g_list_append(toremove, (gpointer)package);
						notfound=FALSE;
					}						
				}
			}			
		}
	}
	
	for (listcounter=0; listcounter<g_list_length(toremove); listcounter++) {		
		GuyumPackageComponent* package = (GuyumPackageComponent*)g_list_nth_data(toremove, listcounter);
		guyum_package_composite_remove_component(self, package);
		guyum_package_component_destroy(package);
	}
	
	g_list_free(toremove);	
}

/**
* remove component
* @param package_composite "this" object
* @param package	GuyumPackageComponent
*/
void guyum_package_composite_remove_component(	GuyumPackageComposite *package_composite,
													GuyumPackageComponent *package) 
{
	g_return_if_fail(package_composite!=NULL);
	g_return_if_fail(package_composite->priv!=NULL);
	g_return_if_fail(package_composite->priv->package_components!=NULL);
	if (g_list_find(package_composite->priv->package_components, (gconstpointer) package)!=NULL) {
		package_composite->priv->package_components = g_list_remove(package_composite->priv->package_components,
																	(gconstpointer) package);	
		package_composite->priv->components_count--;
	} else {
		g_fprintf(stderr, "error: cannot remove package from the list\n");
	}		
}														



/**
* get component count
* @param package_composite "this" object
* @return gint amount of components
*/
gint guyum_package_composite_get_component_count(GuyumPackageComposite *package_composite) 
{
	if (package_composite==NULL) {
		return -1;
	}
	if (package_composite->priv==NULL) {
		return -1;
	}	
	
	return package_composite->priv->components_count;
}

/**
* get nth component from composite or NULL if nth component does not exists
* @param package_composite "this" object
* @param gint number of component
* @return GuyumPackageComponent component
*/
GuyumPackageComponent *guyum_package_composite_get_nth_component(	GuyumPackageComposite *package_composite,
																			gint position) 
{	
	g_return_val_if_fail(GUYUM_IS_PACKAGE_COMPOSITE(package_composite), NULL);
	return (GuyumPackageComponent *)g_list_nth_data(package_composite->priv->package_components, position);
}

/**
* get nth component from composite or NULL if nth component does not exists
* @param package_composite "this" object
* @param gint number of component
* @return GuyumPackageComponent component
*/
GuyumPackageComponent *guyum_package_composite_get_id_component(GuyumPackageComposite *package_composite,
																const gchar* packageid) 
{	
    g_return_val_if_fail(package_composite!=NULL, NULL);
	g_return_val_if_fail(GUYUM_IS_PACKAGE_COMPOSITE(package_composite), NULL);
	g_return_val_if_fail(packageid!=NULL, NULL);
	gint i;
	gboolean cont = TRUE;
	GuyumPackageComponent* package = NULL;
	for (i=0; i<package_composite->priv->components_count && cont; i++) {
		gchar *newid = NULL;
		package = g_list_nth_data(package_composite->priv->package_components, i);
		GOBJECT_GET_STRING_PROPERTY(package, "packageid", &newid);
		if (g_strcasecmp(newid, packageid)==0) {
			cont = FALSE;
		}
		g_free(newid);
	}
	return package;
}
