/***************************************************************************
 *            guyum_query_component.c
 *
 *  $LastChangedDate: 2005-12-26 23:59:21 +0100 (Mo, 26 Dez 2005) $
 *  Copyright  2005  $Author: vklein $
 *  Email
 ****************************************************************************/
#include <include/guyum/gy_query.h>


/****************************************************************************
* Function declarations													
****************************************************************************/
static void guyum_query_component_class_init(GuyumQueryComponentClass *class);

static void guyum_query_component_init(	GuyumQueryComponent *query,
										GuyumQueryComponentClass *class);
														 
static void guyum_query_component_set_property(	GObject *object,
												guint property_id,
												const GValue *value,
												GParamSpec *pspec);
													
static void guyum_query_component_get_property(	GObject *object,
												guint property_id,
												GValue *value,
												GParamSpec *pspec);


/**
* Our "private" properties
*/
enum {
	QUERY_ID=1,
	QUERY_SEARCHSTRING,	
	QUERY_PACKAGES,
	QUERY_REPOSITORIES,
	QUERY_DEPS,
	QUERY_RENEW,
};

/****************************************************************************
* typical functions															
****************************************************************************/
/**
* register new type 
*/
GType guyum_query_component_get_type(void) {
	static GType object_type = 0;
	if (!object_type) {
		static const GTypeInfo object_info = {
			sizeof(GuyumQueryComponentClass),
			NULL,
			NULL,
			(GClassInitFunc)guyum_query_component_class_init,
			NULL,
			NULL,
			sizeof(GuyumQueryComponent),
			0,
			(GInstanceInitFunc)guyum_query_component_init,
			NULL,
		};
		object_type = g_type_register_static(G_TYPE_OBJECT, "GuyumQueryComponent", &object_info, 0);		
	}
	return object_type;
}


/**
* init class
*/
static void guyum_query_component_class_init(GuyumQueryComponentClass *class) {
	GObjectClass *gobject_class = G_OBJECT_CLASS(class);
	GParamSpec *param_spec;

	/* Install features 
	* http://developer.gnome.org/doc/API/2.0/gobject/gobject-properties.html
	* create new objects:
	* 		g_object_new(G_OBJECT(...), "property1", "value1", "property2", value2, ... NULL)
	*/
	gobject_class->set_property = guyum_query_component_set_property;
	gobject_class->get_property = guyum_query_component_get_property;
	
	/* searchstring */
	param_spec = g_param_spec_string("searchstring", 
									"search", 
									"searching for package", 
									"", 
									G_PARAM_READWRITE);	
	g_object_class_install_property(gobject_class, 
									QUERY_SEARCHSTRING, 
									param_spec);
	
	/* id */
	param_spec = g_param_spec_int(	"id", 
									"id", 
									"query id", 
									0, 
									1000000, 
									0, 
									G_PARAM_READWRITE);
	
	g_object_class_install_property(gobject_class, 
									QUERY_ID, 
									param_spec);	

	/* packages */
	param_spec=g_param_spec_pointer("packages", 
									"packagelist", 
									"guyum packages", 
									G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class, 
									QUERY_PACKAGES, 
									param_spec);
	/* deps */
	param_spec=g_param_spec_pointer("deps", 
									"dep list", 
									"guyum package dependencies", 
									G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class, 
									QUERY_DEPS, 
									param_spec);


	/* renew */
	param_spec = g_param_spec_boolean(	"renew", 
									"renew", 
									"query renew", 
									FALSE, 
									G_PARAM_READWRITE);
	
	g_object_class_install_property(gobject_class, 
									QUERY_RENEW, 
									param_spec);	

}

/**
* init instance
*/
static void guyum_query_component_init(GuyumQueryComponent *query,
										 GuyumQueryComponentClass *class) 
{												 
	GuyumQueryComponent *self = (GuyumQueryComponent*)query;
	self->priv = g_new0(GuyumQueryComponentPrivate, 1);
	self->priv->packages = g_object_new(GUYUM_TYPE_PACKAGE_COMPOSITE, NULL);
	self->priv->deps = g_object_new(GUYUM_TYPE_PACKAGE_COMPOSITE, NULL);
	
	self->priv->yumquery_module = PyImport_ImportModule("guyumquery");
	self->priv->yumquery_dict = PyModule_GetDict(self->priv->yumquery_module);
	self->priv->yumquery_class = PyDict_GetItemString(self->priv->yumquery_dict, "GuyumQuery");
	
	if (PyCallable_Check(self->priv->yumquery_class)) 
	{
		self->priv->yumquery_instance = PyObject_CallObject(self->priv->yumquery_class, NULL);
	}
	
	/* init object */
	PyObject* py_init = PyObject_CallMethod(self->priv->yumquery_instance, "init", NULL);
	Py_DECREF(py_init);	
	
}


/**
* set properties
*/
static void guyum_query_component_set_property(	GObject *object,
												guint property_id,
												const GValue *value,
												GParamSpec *pspec) {
	GuyumQueryComponent *self = (GuyumQueryComponent*)object;
										
	switch(property_id) {
		case QUERY_SEARCHSTRING: {
			if (self->priv->searchstring!=NULL) {
				g_free(self->priv->searchstring);
			}
			self->priv->searchstring = g_value_dup_string(value);			
		}
		break;		

		case QUERY_ID: {
			self->priv->id = g_value_get_int(value);			
		}
		break;						
		case QUERY_PACKAGES: {
			if (self->priv->packages!=NULL) {
				guyum_package_composite_destroy(self->priv->packages);			
				self->priv->packages = NULL;
			}
			self->priv->packages = g_value_get_pointer(value);			
		}
		break;
		case QUERY_DEPS: {
			if (self->priv->deps!=NULL) {
				guyum_package_composite_destroy(self->priv->deps);			
				self->priv->deps = NULL;
			}
			self->priv->deps = g_value_get_pointer(value);			
		}
		break;		
		case QUERY_RENEW: {
			self->priv->renew = g_value_get_boolean(value);
		}
		break;						
		
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object,property_id,pspec);
    		break;
  	}
		
}

/**
* get properties
*/
static void guyum_query_component_get_property(	GObject *object,
													guint property_id,
													GValue *value,
													GParamSpec *pspec) {	
	GuyumQueryComponent *self = (GuyumQueryComponent*)object;
														
	switch(property_id) {
		case QUERY_SEARCHSTRING: {
			g_value_set_string(value, self->priv->searchstring);			
		}
		break;		
		case QUERY_ID: {
			g_value_set_int(value, self->priv->id);
		}
		break;	
		
		case QUERY_PACKAGES: {
			g_value_set_pointer(value, self->priv->packages);
		}
		break;	
		case QUERY_DEPS: {
			g_value_set_pointer(value, self->priv->deps);
		}
		break;
		case QUERY_RENEW: {
			g_value_set_boolean(value, self->priv->renew);
		}
		break;
		
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object,property_id,pspec);
    		break;
  	}	
}

/**
*
* destructor
*/
void guyum_query_component_destroy(GuyumQueryComponent *query) {
	g_return_if_fail(query!=NULL);
	g_return_if_fail(GUYUM_IS_QUERY_COMPONENT(query));
	if (query->priv!=NULL) {
		if (query->priv->packages!=NULL) {
			guyum_package_composite_destroy(query->priv->packages);
		}
		if (query->priv->deps!=NULL) {
			guyum_package_composite_destroy(query->priv->deps);
		}
		if (query->priv->searchstring!=NULL) {
			g_free(query->priv->searchstring);
		}
		/* clean up python stuff */
		if (query->priv->yumquery_deps!=NULL && PyCallable_Check(query->priv->yumquery_deps)) {
			Py_DECREF(query->priv->yumquery_deps);
		}
		if (query->priv->yumquery_packages!=NULL && PyCallable_Check(query->priv->yumquery_packages)) {
			Py_DECREF(query->priv->yumquery_packages);
		}
		Py_DECREF(query->priv->yumquery_module);
		Py_DECREF(query->priv->yumquery_instance);		
		g_free(query->priv);
	}
}


/****************************************************************************
* query component functions												
****************************************************************************/

/**
*
*/
gy_status_t
guyum_query_component_search(	GuyumQueryComponent* self, 
								const gchar* query, 
								GList* searchlist, 
								gchar* repoid)
{

	int i = 0;
	gy_status_t gyrv = GY_SUCCESS;
	g_return_val_if_fail(self!=NULL, GY_SELF_IS_NULL);
	g_return_val_if_fail(GUYUM_IS_QUERY_COMPONENT(self), GY_TYPE_FAILED);
    g_return_val_if_fail(repoid!=NULL, GY_TYPE_FAILED);

	/* Create searchfieldlist */
	PyObject *py_searchlist = PyList_New(0);
	for (i=0; i<g_list_length(searchlist); i++) {
		gchar* searchfield = (gchar*)g_list_nth_data(searchlist, i);
		PyList_Append(py_searchlist, PyString_FromString(searchfield));
	}	

	/* Create repolist */
	GOBJECT_SET_STRING_PROPERTY(self, "searchstring", query);
	if (!(self->priv->yumquery_packages = PyObject_CallMethod(self->priv->yumquery_instance, 
																"searchpackage", 
																"(sOs)", 
																self->priv->searchstring, 
																py_searchlist, 
																repoid))) {
		Py_DECREF(py_searchlist);	
		return GY_QUERY_FAILED;																	
	}

	if (self->priv->yumquery_packages!=NULL && PyList_Check(self->priv->yumquery_packages)) {

		/* Iterate packages. Packages are dictionaries in a list */
		for (i=0; i<PyList_Size(self->priv->yumquery_packages); i++) {
			GuyumPackageComponent* guyum_package;
			gboolean canadd;
			
			PyObject *py_package = PyList_GetItem(self->priv->yumquery_packages, i);
			
			guyum_package = g_object_new(GUYUM_TYPE_PACKAGE_COMPONENT, 
										"arch", PyString_AsString(PyObject_GetAttrString(py_package, "arch")),
										"changelog", PyString_AsString(PyObject_GetAttrString(py_package, "changelog")),
										"compsize", PyString_AsString(PyObject_GetAttrString(py_package, "compsize")),
										"conflicts", Py_List_As_GList(PyObject_GetAttrString(py_package, "conflicts")),
										"description", PyString_AsString(PyObject_GetAttrString(py_package, "description")),
										"epoch", PyString_AsString(PyObject_GetAttrString(py_package, "epoch")),
										"files", Py_List_As_GList(PyObject_GetAttrString(py_package, "files")),
										"group", PyString_AsString(PyObject_GetAttrString(py_package, "group")),
										"licenses", PyString_AsString(PyObject_GetAttrString(py_package, "licenses")),
										"localrelease", PyString_AsString(PyObject_GetAttrString(py_package, "localrelease")),
										"localversion", PyString_AsString(PyObject_GetAttrString(py_package, "localversion")),
										"name", PyString_AsString(PyObject_GetAttrString(py_package, "name")),
										"obsoletes", Py_List_As_GList(PyObject_GetAttrString(py_package, "obsoletes")),
										"packageid",PyString_AsString(PyObject_GetAttrString(py_package, "packageid")),
										"pkgsize", PyString_AsString(PyObject_GetAttrString(py_package, "compsize")),
										"provides", Py_List_As_GList(PyObject_GetAttrString(py_package, "provides")),
										"release", PyString_AsString(PyObject_GetAttrString(py_package, "release")),
										"remoterelease", PyString_AsString(PyObject_GetAttrString(py_package, "remoterelease")),			
										"remoteversion", PyString_AsString(PyObject_GetAttrString(py_package, "remoteversion")),
										"repoid", PyString_AsString(PyObject_GetAttrString(py_package, "repoid")),
										"requires", Py_List_As_GList(PyObject_GetAttrString(py_package, "requires")),
										"state", PyString_AsString(PyObject_GetAttrString(py_package, "state")),
										"summary", PyString_AsString(PyObject_GetAttrString(py_package, "summary")),
										"url", PyString_AsString(PyObject_GetAttrString(py_package, "url")),
										"version", PyString_AsString(PyObject_GetAttrString(py_package, "version")),
										"vendor", PyString_AsString(PyObject_GetAttrString(py_package, "vendor")),
										
										//"yumpackage", py_package,
										NULL
										);

				guyum_package_composite_add_component(self->priv->packages, guyum_package, &canadd);				
			
		}

	} else {	
		//g_printf("else: 4000\n");
		GUYUM_PYTHON_FATAL_ERROR();
	}
	Py_DECREF(py_searchlist);	
	guyum_package_composite_remove_doubles(self->priv->packages);
	return gyrv;
}

/**
*
*/
void 
guyum_query_component_rpmdbsearch(GuyumQueryComponent* self, gchar* repoid)
{
	gint i =0;

	PyObject *py_searchlist = PyList_New(0);
	PyObject *searchresult;
	/* Create python list from packages */
	for (i=0; i<guyum_package_composite_get_component_count(self->priv->packages); i++) {
		gchar* name = NULL;
		gchar* localversion = NULL;
		gchar* remoteversion = NULL;
		gchar* packageid = NULL;
		gchar* localrelease = NULL;
		gchar* remoterelease = NULL;
		
		GuyumPackageComponent* package = guyum_package_composite_get_nth_component(self->priv->packages, i);
		
		PyObject *py_package = PyTuple_New(6);
		
		GOBJECT_GET_STRING_PROPERTY(package, "name", &name);		
		GOBJECT_GET_STRING_PROPERTY(package, "version", &localversion);		
		GOBJECT_GET_STRING_PROPERTY(package, "remoteversion", &remoteversion);
		GOBJECT_GET_STRING_PROPERTY(package, "packageid", &packageid);
		GOBJECT_GET_STRING_PROPERTY(package, "release", &localrelease);
		GOBJECT_GET_STRING_PROPERTY(package, "remoterelease", &remoterelease);
		
		PyTuple_SetItem(py_package, 0, PyString_FromString(name));
		PyTuple_SetItem(py_package, 1, PyString_FromString(localversion));
		PyTuple_SetItem(py_package, 2, PyString_FromString(localrelease));
		PyTuple_SetItem(py_package, 3, PyString_FromString(remoteversion));
		PyTuple_SetItem(py_package, 4, PyString_FromString(remoterelease));
		PyTuple_SetItem(py_package, 5, PyString_FromString(packageid));
		
		/* Create searchfieldlist */	
		PyList_Append(py_searchlist, py_package);
		
		Py_DECREF(py_package);
		
		g_free(name);
		g_free(localversion);
		g_free(remoteversion);
		g_free(packageid);
		g_free(localrelease);
		g_free(remoterelease);
		
	}
	/* Call python function */
	searchresult = PyObject_CallMethod(self->priv->yumquery_instance, "rpmdbsearch", "(O)", py_searchlist);
	if (searchresult!=NULL && PyList_Check(searchresult)) {
		/* Iterate packages. Packages are dictionaries in a list */
		for (i=0; i<PyList_Size(searchresult); i++) {
			GuyumPackageComponent* guyum_package;
			gchar* packageid;
			gchar* state;
			gchar* newversion;
			gchar* newrelease;
            gchar* packagerepo;
			PyObject *py_package = PyList_GetItem(searchresult, i);
			packageid = PyString_AsString(PyTuple_GetItem(py_package, 0));
			newversion = PyString_AsString(PyTuple_GetItem(py_package, 1));
			newrelease = PyString_AsString(PyTuple_GetItem(py_package, 2));
			state = PyString_AsString(PyTuple_GetItem(py_package, 3));

			guyum_package = guyum_package_composite_get_id_component(self->priv->packages, (const gchar*)packageid);            
			if (guyum_package!=NULL) {
                GOBJECT_GET_STRING_PROPERTY(guyum_package, "repoid", &packagerepo);
                if (g_strcasecmp(packagerepo, repoid)==0) {
                    GOBJECT_SET_STRING_PROPERTY(guyum_package, "state", state);
                    GOBJECT_SET_STRING_PROPERTY(guyum_package, "localversion", newversion);
                    GOBJECT_SET_STRING_PROPERTY(guyum_package, "localrelease", newrelease);
                }
			} else {			
				// g_printf("----guyum_package nicht da\n");
			}		
			
		}
		Py_DECREF(searchresult);
    	}
	Py_DECREF(py_searchlist);

}





/**
*
*/
void
guyum_query_setlog(GuyumQueryComponent* query, gchar* filename, gint threshold)
{
	PyObject* retvalue;
	g_return_if_fail(query!=NULL);
	g_return_if_fail(filename!=NULL);

	retvalue = PyObject_CallMethod(query->priv->yumquery_instance, "setlog", "(si)", filename, threshold);
	Py_DECREF(retvalue);

}

/**
*
*/
PyObject*
guyum_query_component_get_package(GuyumQueryComponent* self,
									const gchar* packageid)
{
	PyObject* package;
	g_return_val_if_fail(self!=NULL, NULL);
	g_return_val_if_fail(GUYUM_IS_QUERY_COMPONENT(self), NULL);
	
	package = PyObject_CallMethod(self->priv->yumquery_instance, "findpackage", "(s)", packageid);	
	
	return package;

}
