/***************************************************************************
 *            guyum_query_composite.c
 *
 *  $LastChangedDate: 2005-12-31 11:25:00 +0100 (Sa, 31 Dez 2005) $
 *  Copyright  2005  $Author: vklein $
 *  Email
 ****************************************************************************/
#include <include/guyum/gy_query_list.h>



/****************************************************************************
* Function declarations													
****************************************************************************/

static void guyum_query_composite_init(GuyumQueryComposite *query,
										GuyumQueryCompositeClass *class);
														 


/****************************************************************************
* typical functions															
****************************************************************************/
/**
* register new type 
*/
GType guyum_query_composite_get_type(void) 
{
	static GType object_type = 0;
	if (!object_type) {
		static const GTypeInfo object_info = {
			sizeof(GuyumQueryCompositeClass),
			NULL,
			NULL,
			NULL,
			NULL,
			NULL,
			sizeof(GuyumQueryComposite),
			0,
			(GInstanceInitFunc)guyum_query_composite_init,
			NULL,
		};
		object_type = g_type_register_static(G_TYPE_OBJECT, "GuyumQueryComposite", &object_info, 0);		
	}
	return object_type;
}

/**
* init instance
*/
static void guyum_query_composite_init(GuyumQueryComposite *query,
										 GuyumQueryCompositeClass *class) 
{	
											 
	GuyumQueryComposite *self = (GuyumQueryComposite*)query;
	self->priv = g_new0(GuyumQueryCompositePrivate, 1);
	self->priv->query_components = NULL;
	self->priv->components_count = 0;
											 
}

/**
*
* destructor
*/
void guyum_query_composite_destroy(GuyumQueryComposite *query_composite) 
{
	g_return_if_fail(query_composite!=NULL);
	g_return_if_fail(GUYUM_IS_QUERY_COMPOSITE(query_composite));
	if (query_composite->priv!=NULL) {
		if (query_composite->priv->query_components!=NULL) {
			gint i = 0;
			for (; i<query_composite->priv->components_count; i++) {
				guyum_query_component_destroy(guyum_query_composite_get_nth_component(query_composite, i));				
			}
			g_list_free(query_composite->priv->query_components);
		}			
		g_free(query_composite->priv);
	}
}

/****************************************************************************/
/* composite functions														*/
/****************************************************************************/

/**
* add new query component to query composite
* @param query_composite "this" object
* @param query	GuyumQueryComponent
*/
void guyum_query_composite_add_component(	GuyumQueryComposite *query_composite,
												GuyumQueryComponent *query) 
{
	g_return_if_fail(query_composite!=NULL);
	g_return_if_fail(query_composite->priv!=NULL);
	query_composite->priv->query_components = g_list_append(query_composite->priv->query_components, 
																(gpointer)query);	
	query_composite->priv->components_count++;
}


/**
* insert new query component on position pos
* @param query_composite "this" object
* @param query	GuyumQueryComponent
*/
void guyum_query_composite_insert_component(	GuyumQueryComposite *query_composite,
												GuyumQueryComponent *query,
												gint pos) 
{
	g_return_if_fail(query_composite!=NULL);
	g_return_if_fail(query_composite->priv!=NULL);
	query_composite->priv->query_components = g_list_insert(query_composite->priv->query_components, 
															(gpointer)query,
															pos);
	query_composite->priv->components_count++;
}

/**
* remove component
* @param query_composite "this" object
* @param query	GuyumQueryComponent
*/
void guyum_query_composite_remove_component(	GuyumQueryComposite *query_composite,
													GuyumQueryComponent *query) 
{
	g_return_if_fail(query_composite!=NULL);
	g_return_if_fail(query_composite->priv!=NULL);
	g_return_if_fail(query_composite->priv->query_components!=NULL);
	
	if (g_list_find(query_composite->priv->query_components, (gconstpointer) query)!=NULL) {
		query_composite->priv->query_components = g_list_remove(query_composite->priv->query_components,
																	(gconstpointer) query);	
		guyum_query_component_destroy(query);
		query_composite->priv->components_count--;
	}																
}														

/**
* get component count
* @param query_composite "this" object
* @return gint amount of components
*/
gint guyum_query_composite_get_component_count(GuyumQueryComposite *query_composite) 
{
	if (query_composite==NULL) {
		return -1;
	}
	if (query_composite->priv==NULL) {
		return -1;
	}	
	
	return query_composite->priv->components_count;
}

/**
* get nth component from composite or NULL if nth component does not exists
* @param query_composite "this" object
* @param gint number of component
* @return GuyumQueryComponent component
*/
GuyumQueryComponent *guyum_query_composite_get_nth_component(	GuyumQueryComposite *self,
																			gint position) 
{
	g_return_val_if_fail(self!=NULL, NULL);
	g_return_val_if_fail(GUYUM_IS_QUERY_COMPOSITE(self), NULL);
	g_return_val_if_fail(position>=0, NULL);
	g_return_val_if_fail(position<g_list_length(self->priv->query_components), NULL);
	
	return (GuyumQueryComponent *)g_list_nth_data(self->priv->query_components, position);
}


/**
*
*/
GuyumPackageComposite* guyum_query_composite_execute_component(	GuyumQueryComposite *query_composite, 
																GuyumQueryComponent *query, 
																GList* searchlist, 
																gchar* repoid)
{
	GuyumPackageComposite* packages = NULL;
	
	g_return_val_if_fail(query_composite!=NULL, NULL);
	g_return_val_if_fail(GUYUM_IS_QUERY_COMPOSITE(query_composite), NULL);
	if (g_list_find(query_composite->priv->query_components, (gconstpointer) query)!=NULL) {
		guyum_query_component_search(query, "", searchlist, repoid);		

		GOBJECT_GET_POINTER_PROPERTY(query, "packages", packages);
	}
	return packages;
}

void
guyum_query_composite_set_renew(GuyumQueryComposite* self, gint currentquery)
{
	g_return_if_fail(self!=NULL);
	g_return_if_fail(GUYUM_IS_QUERY_COMPOSITE(self));
	gint i = 0;
	for (; i<self->priv->components_count; i++) {
		if (i!=currentquery) {
			GuyumQueryComponent* query = guyum_query_composite_get_nth_component(self, i);				
			GOBJECT_SET_BOOLEAN_PROPERTY(query, "renew");
		}
	}	
}


gboolean
guyum_query_composite_get_renew(GuyumQueryComposite* self, gint currentquery)
{
	g_return_val_if_fail(self!=NULL, FALSE);
	g_return_val_if_fail(GUYUM_IS_QUERY_COMPOSITE(self), FALSE);
	gboolean retvalue = FALSE;
	GuyumQueryComponent* query = guyum_query_composite_get_nth_component(self, currentquery);				
	GOBJECT_GET_BOOLEAN_PROPERTY(query, "renew", &retvalue);
	return retvalue;
}
