/***************************************************************************
 *            guyum_repo_component.c
 *
 *  $LastChangedDate: 2005-12-26 23:59:21 +0100 (Mo, 26 Dez 2005) $
 *  Copyright  2005  $Author: vklein $
 *  Email
 ****************************************************************************/
#include <include/guyum/gy_repo.h>


/****************************************************************************
* Function declarations													
****************************************************************************/
static void guyum_repo_component_class_init(GuyumRepoComponentClass *class);

static void guyum_repo_component_init(	GuyumRepoComponent *repo,
										GuyumRepoComponentClass *class);
														 
static void guyum_repo_component_set_property(	GObject *object,
												guint property_id,
												const GValue *value,
												GParamSpec *pspec);
													
static void guyum_repo_component_get_property(	GObject *object,
												guint property_id,
												GValue *value,
												GParamSpec *pspec);


/**
* Our "private" properties
*/
enum {
	REPO_ID=1,
	REPO_NAME,
	REPO_ENABLED,
};

/****************************************************************************
* typical functions															
****************************************************************************/
/**
* register new type 
*/
GType guyum_repo_component_get_type(void) {
	static GType object_type = 0;
	if (!object_type) {
		static const GTypeInfo object_info = {
			sizeof(GuyumRepoComponentClass),
			NULL,
			NULL,
			(GClassInitFunc)guyum_repo_component_class_init,
			NULL,
			NULL,
			sizeof(GuyumRepoComponent),
			0,
			(GInstanceInitFunc)guyum_repo_component_init,
			NULL,
		};
		object_type = g_type_register_static(G_TYPE_OBJECT, "GuyumRepoComponent", &object_info, 0);		
	}
	return object_type;
}


/**
* init class
*/
static void guyum_repo_component_class_init(GuyumRepoComponentClass *class) {
	GObjectClass *gobject_class = G_OBJECT_CLASS(class);
	GParamSpec *param_spec;

	/* Install features 
	* http://developer.gnome.org/doc/API/2.0/gobject/gobject-properties.html
	* create new objects:
	* 		g_object_new(G_OBJECT(...), "property1", "value1", "property2", value2, ... NULL)
	*/
	gobject_class->set_property = guyum_repo_component_set_property;
	gobject_class->get_property = guyum_repo_component_get_property;
	
	/* id */
	param_spec = g_param_spec_string("id", 
									"repoid", 
									"repository id", 
									"", 
									G_PARAM_READWRITE);	
	g_object_class_install_property(gobject_class, 
									REPO_ID, 
									param_spec);
	
	/* name */
	param_spec = g_param_spec_string("name", 
									"reponame", 
									"repository name", 
									"", 
									G_PARAM_READWRITE);	
	g_object_class_install_property(gobject_class, 
									REPO_NAME, 
									param_spec);	
									
	/* enabled */
	param_spec = g_param_spec_boolean(	"enabled", 
										"repoenabled", 
										"repository enabled", 
										FALSE, 
										G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class, 
									REPO_ENABLED, 
									param_spec);
																		
}

/**
* init instance
*/
static void guyum_repo_component_init(GuyumRepoComponent *repo,
										 GuyumRepoComponentClass *class) 
{												 
	GuyumRepoComponent *self = (GuyumRepoComponent*)repo;
	self->priv = g_new0(GuyumRepoComponentPrivate, 1);
}


/**
* set properties
*/
static void guyum_repo_component_set_property(	GObject *object,
												guint property_id,
												const GValue *value,
												GParamSpec *pspec) {
	GuyumRepoComponent *self = (GuyumRepoComponent*)object;
										
	switch(property_id) {
		case REPO_ID: {
			if (self->priv->id!=NULL) {
				g_free(self->priv->id);
			}
			self->priv->id = g_value_dup_string(value);			
		}
		break;	
		case REPO_NAME: {
			if (self->priv->name!=NULL) {
				g_free(self->priv->name);
			}
			self->priv->name = g_value_dup_string(value);			
		}
		break;		
		case REPO_ENABLED: {
			self->priv->enabled = g_value_get_boolean(value);
		}
		break;		

		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object,property_id,pspec);
    		break;
  	}
		
}

/**
* get properties
*/
static void guyum_repo_component_get_property(	GObject *object,
													guint property_id,
													GValue *value,
													GParamSpec *pspec) 
{	
	GuyumRepoComponent *self = (GuyumRepoComponent*)object;

	switch(property_id) {			
		case REPO_ID: {
			g_value_set_string(value, self->priv->id);
		}
		break;		
		case REPO_NAME: {
			g_value_set_string(value, self->priv->name);
		}
		break;		
		case REPO_ENABLED: {
			g_value_set_boolean(value, self->priv->enabled);
		}
		break;		
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object,property_id,pspec);
    		break;
  	}	
}

/**
*
* destructor
*/
void guyum_repo_component_destroy(GuyumRepoComponent *repo) {
	g_return_if_fail(repo!=NULL);
	g_return_if_fail(GUYUM_IS_REPO_COMPONENT(repo));
	if (repo->priv!=NULL) {		
		g_free(repo->priv->id);		
		g_free(repo->priv->name);
		g_free(repo->priv);		
	}
}
