/***************************************************************************
 *            guyum_repo_composite.c
 *
 *  $LastChangedDate: 2005-12-31 11:25:00 +0100 (Sa, 31 Dez 2005) $
 *  Copyright  2005  $Author: vklein $
 *  Email
 ****************************************************************************/
#include <include/guyum/gy_repo_list.h>



/****************************************************************************
* Function declarations													
****************************************************************************/

static void guyum_repo_composite_init(GuyumRepoComposite *repo,
										GuyumRepoCompositeClass *class);
														 


/****************************************************************************
* typical functions															
****************************************************************************/
/**
* register new type 
*/
GType guyum_repo_composite_get_type(void) 
{
	static GType object_type = 0;
	if (!object_type) {
		static const GTypeInfo object_info = {
			sizeof(GuyumRepoCompositeClass),
			NULL,
			NULL,
			NULL,
			NULL,
			NULL,
			sizeof(GuyumRepoComposite),
			0,
			(GInstanceInitFunc)guyum_repo_composite_init,
			NULL,
		};
		object_type = g_type_register_static(G_TYPE_OBJECT, "GuyumRepoComposite", &object_info, 0);		
	}
	return object_type;
}

/**
* init instance
*/
static void guyum_repo_composite_init(GuyumRepoComposite *repo,
										 GuyumRepoCompositeClass *class) 
{	
											 
	GuyumRepoComposite *self = (GuyumRepoComposite*)repo;
	self->priv = g_new0(GuyumRepoCompositePrivate, 1);
	self->priv->repo_components = NULL;
	self->priv->components_count = 0;
											 
}

/**
*
* destructor
*/
void guyum_repo_composite_destroy(GuyumRepoComposite *repo_composite) 
{
	g_return_if_fail(repo_composite!=NULL);
	g_return_if_fail(GUYUM_IS_REPO_COMPOSITE(repo_composite));
	if (repo_composite->priv!=NULL) {
		if (repo_composite->priv->repo_components!=NULL) {
			gint i = 0;
			for (; i<repo_composite->priv->components_count; i++) {
				guyum_repo_component_destroy(guyum_repo_composite_get_nth_component(repo_composite, i));				
			}
			g_list_free(repo_composite->priv->repo_components);
		}			
		g_free(repo_composite->priv);
	}
}

/****************************************************************************/
/* composite functions														*/
/****************************************************************************/

/**
* add new repo component to repo composite
* @param repo_composite "this" object
* @param repo	GuyumRepoComponent
*/
void guyum_repo_composite_add_component(	GuyumRepoComposite *repo_composite,
												GuyumRepoComponent *repo) 
{
	g_return_if_fail(repo_composite!=NULL);
	g_return_if_fail(repo_composite->priv!=NULL);
	repo_composite->priv->repo_components = g_list_append(repo_composite->priv->repo_components, 
																(gpointer)repo);
	repo_composite->priv->components_count++;
}

/**
* remove component
* @param repo_composite "this" object
* @param repo	GuyumRepoComponent
*/
void guyum_repo_composite_remove_component(	GuyumRepoComposite *repo_composite,
													GuyumRepoComponent *repo) 
{
	g_return_if_fail(repo_composite!=NULL);
	g_return_if_fail(repo_composite->priv!=NULL);
	g_return_if_fail(repo_composite->priv->repo_components!=NULL);
	if (g_list_find(repo_composite->priv->repo_components, (gconstpointer) repo)!=NULL) {
		repo_composite->priv->repo_components = g_list_remove(repo_composite->priv->repo_components,
																	(gconstpointer) repo);	
		repo_composite->priv->components_count--;
	}																
}														

/**
* get component count
* @param repo_composite "this" object
* @return gint amount of components
*/
gint guyum_repo_composite_get_component_count(GuyumRepoComposite *repo_composite) 
{
	if (repo_composite==NULL) {
		return -1;
	}
	if (repo_composite->priv==NULL) {
		return -1;
	}	
	
	return repo_composite->priv->components_count;
}

/**
* get nth component from composite or NULL if nth component does not exists
* @param repo_composite "this" object
* @param gint number of component
* @return GuyumRepoComponent component
*/
GuyumRepoComponent *guyum_repo_composite_get_nth_component(	GuyumRepoComposite *repo_composite,
																			gint position) 
{
	g_return_val_if_fail(GUYUM_IS_REPO_COMPOSITE(repo_composite), NULL);
	return (GuyumRepoComponent *)g_list_nth_data(repo_composite->priv->repo_components, position);
}

/**
*
*/
GuyumRepoComponent *guyum_repo_composite_get_named_component(	GuyumRepoComposite *repo_composite,
																const gchar* repoid) 
{
	g_return_val_if_fail(GUYUM_IS_REPO_COMPOSITE(repo_composite), NULL);
	GuyumRepoComponent *retcomponent = NULL;
	int i =0;
	for (i=0; i<guyum_repo_composite_get_component_count(repo_composite) && retcomponent==NULL; i++) {
		GuyumRepoComponent *component = guyum_repo_composite_get_nth_component(repo_composite, i);
		gchar* reponame;
		GOBJECT_GET_STRING_PROPERTY(G_OBJECT(component), "id", &reponame);
		
		if (g_strcasecmp(repoid, reponame)==0) {
			retcomponent = component;
		}
	}
	return retcomponent;
}
