#include <include/install.h>

extern GuyumMain* guyummain;
extern GladeXML* xml;
extern GtkWidget* window1;
extern GtkWidget* logdialog;
extern GdkCursor* cursor;
extern GtkWidget* depsdialog;
extern GtkWidget* installokdialog;

extern GAsyncQueue* asyncqueue;


/**
*
*/
gboolean
execute_install(gpointer *user_data)
{
	gchar* pkgid;
	GtkNotebook* notebook_query;
	
	pkgid = (gchar*)user_data;	

    notebook_query = GTK_NOTEBOOK(glade_xml_get_widget(xml, "guyum_notebook_query"));				
    guyum_main_execute_install(guyummain, gtk_notebook_get_current_page(notebook_query));			
    g_idle_add(install_notify, NULL);
	
	return FALSE;
	
}

/**
*
*/
gboolean
run_preinstall_thread(gpointer *user_data)
{
	gchar* pkgid;
	GtkNotebook* notebook_query;
	GtkNotebook* deps_notebook_query;
	GtkWidget* child;
	int pageindex;
	GtkTreeView* treeview;
	GtkTreeSelection* selection;
	GtkLabel* amount;
	GString* totalamount = NULL;
	GtkTreeIter treepath;
	GuyumPackageComposite* deps = NULL;
	
	/* Package ID to install */
	pkgid = (gchar*)user_data;
	
	notebook_query = GTK_NOTEBOOK(glade_xml_get_widget(xml, "guyum_notebook_query"));
	deps = guyum_main_finddeps(guyummain, gtk_notebook_get_current_page(notebook_query), pkgid);
	
	if (deps!=NULL) {
		
		/* depsdialog -> TK */
		depsdialog = GTK_WIDGET(glade_xml_get_widget(xml, "guyum_dialog_deps"));
		
		deps_notebook_query = GTK_NOTEBOOK(glade_xml_get_widget(xml, "deps_notebook_packages"));
		
		gtk_notebook_remove_page(deps_notebook_query, 0);
				
		child = gtk_scrolled_window_new(NULL, NULL);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(child), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
		
		treeview = create_deppackages_treeview();
		gtk_widget_show(GTK_WIDGET(treeview));
		gtk_container_add(GTK_CONTAINER(child), GTK_WIDGET(treeview));	
		gtk_widget_show(GTK_WIDGET(child));		
	
		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
		gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
		g_signal_connect(G_OBJECT(selection), "changed", G_CALLBACK(deps_selection_changed), deps);		
	
		append_textview_log("Searching for dependencies...\n");
		pageindex = gtk_notebook_append_page(deps_notebook_query, child, gtk_label_new(_("Installation dependencies")));
			
		deppackages_create_and_fill_model(deps, treeview);	
		
		amount = (GtkLabel*)glade_xml_get_widget(xml, "deps_label_totalamount");
		totalamount = g_string_new(_("Total download size: "));
		totalamount = g_string_append(totalamount, guyum_main_get_totalamount(guyummain, gtk_notebook_get_current_page(notebook_query)));
	
		gtk_label_set_text(amount, totalamount->str);
	
		/* select first entry in list */
		if (gtk_tree_model_get_iter_first(gtk_tree_view_get_model(treeview), &treepath)) {	
			gtk_tree_selection_select_iter(selection, &treepath);
		}
		append_textview_log("Done\n");
		gdk_threads_enter();
		HIDE_INSTALLWAIT_DIALOG();
		RESET_CURSOR;
		
		g_signal_connect(G_OBJECT(depsdialog), "destroy", cancel_depsdialog, NULL);
		g_signal_connect(G_OBJECT(depsdialog), "delete_event", cancel_depsdialog, NULL);
		
		gtk_widget_show(depsdialog);	
		gdk_flush();
		gdk_threads_leave();
		
	} else {

		gdk_threads_enter();
		HIDE_INSTALLWAIT_DIALOG();
		RESET_CURSOR;
		
		show_error_dialog(_("Error: Cannot resolve dependencies!"));

		gdk_flush();
		gdk_threads_leave();	
	}

	if (totalamount!=NULL) {
		g_string_free(totalamount, TRUE);	
	}

	return FALSE;
	
}

/**
*
*/
gboolean
install_notify(gpointer user_data)
{

	GuyumQueryComposite* queries;
	GuyumPackageComposite* packages;
	GuyumQueryComponent* query ;
    GuyumInstallComponent* install;
    gchar* repo;
	
	GtkNotebook* notebook_query;
	gchar* searchtext = NULL;	
	gint currentpage = -1;

	gdk_threads_enter();	
	gtk_dialog_run(GTK_DIALOG(installokdialog));
	gdk_flush();
	gdk_threads_leave();
	gtk_widget_hide(installokdialog);		
		
	/* After installation, rerun search in local rpm db */

	notebook_query = (GtkNotebook*)glade_xml_get_widget(xml, "guyum_notebook_query");
	GOBJECT_GET_POINTER_PROPERTY(guyummain, "queries", queries);
    GOBJECT_GET_POINTER_PROPERTY(guyummain, "install", install);	

	query = guyum_query_composite_get_nth_component(queries, gtk_notebook_get_current_page(notebook_query));
    
	/* Run local search */
    GOBJECT_GET_STRING_PROPERTY(G_OBJECT(install), "repoid", &repo);
    
	packages = guyum_main_rpmdbsearch(guyummain, gtk_notebook_get_current_page(notebook_query), repo);
	/* Remove current page */
	currentpage = gtk_notebook_get_current_page(notebook_query);
	gtk_notebook_remove_page(notebook_query, currentpage);
	/* Add new page */
	add_packages_tab(query, currentpage);
	
	gdk_threads_enter();
	gtk_widget_hide(depsdialog);
	gdk_flush();
	gdk_threads_leave();
	
	RESET_CURSOR;	
	HIDE_WAIT_DIALOG();	

	//append_textview_log("Done.\n");
	
	/* Free variables */
	g_free(searchtext);
    g_free(repo);
	return FALSE;
}


/**
* dialog thread
*/
void
run_installwait_dialog_thread(gpointer data) {
	guint sid;
	gdk_threads_enter();
	
	sid = g_timeout_add(200, update_progressbar, glade_xml_get_widget(xml, "dialog_installwait_progressbar"));
	g_object_set_data(G_OBJECT(glade_xml_get_widget(xml, "guyum_dialog_installwait")), "source_id", GINT_TO_POINTER(sid));	
	SET_CURSOR;
	SHOW_INSTALLWAIT_DIALOG((gchar*)data);	
	gdk_flush();
	gdk_threads_leave();			
	g_free(data);
}
