
#include <include/search.h>

extern GuyumMain* guyummain;
extern GladeXML* xml;
extern GtkWidget* window1;
extern GtkWidget* logdialog;
extern GdkCursor* cursor;
extern GtkWidget* depsdialog;
extern GtkWidget* installokdialog;

extern GAsyncQueue* asyncqueue;

/**
* pulse progressbar
*/
gboolean 
update_progressbar(gpointer data)
{
	gtk_progress_bar_pulse(GTK_PROGRESS_BAR(data));
	return TRUE;
}

/**
* dialog thread
*/
void
run_dialog_thread(gpointer data) {
	guint sid;
	gdk_threads_enter();
	
	sid = g_timeout_add(200, update_progressbar, glade_xml_get_widget(xml, "dialog_wait_progressbar"));
	g_object_set_data(G_OBJECT(glade_xml_get_widget(xml, "guyum_dialog_wait")), "source_id", GINT_TO_POINTER(sid));	
	SET_CURSOR;
	SHOW_WAIT_DIALOG((gchar*)data);	
	gdk_flush();
	gdk_threads_leave();			
	
}


/**
* Search thread notify function
*/
gboolean
search_notify(gpointer data)
{
	RESET_CURSOR;	
	HIDE_WAIT_DIALOG();	
	add_packages_tab((GuyumQueryComponent*)data, -1);
	
	g_async_queue_unref(asyncqueue);
	/* free variables */	

	return FALSE;
}


/**
* Search function
*/
gboolean
search_thread(gpointer data)
{
	
	GList* searchfields = get_searchfields();
	GList* repositories = get_repositories();	
	gint repocounter;
	gint cancelsearch = 0;
	gchar* searchtext = (gchar*)data;
	GtkNotebook* notebook_query = GTK_NOTEBOOK(glade_xml_get_widget(xml, "guyum_notebook_query"));
	
	gint position = gtk_notebook_get_n_pages(notebook_query)+1;
	
	GuyumPackageComposite* packages = NULL;
	GuyumQueryComponent* query = guyum_main_init_query(guyummain, position);	

	GTimer* timer = g_timer_new();
	
	for (repocounter=0; repocounter<g_list_length(repositories) && cancelsearch==0; repocounter++) {
		cancelsearch = (gint)g_async_queue_try_pop(asyncqueue);
		if (cancelsearch!=1) {
			
			gchar* repoid = g_list_nth_data(repositories, repocounter);
			GString* waitstring = g_string_new("");
			gulong microseconds = 0;
			
			g_string_printf(waitstring, _("Searching in %s...\n"), repoid);	
			
			gdk_threads_enter();
			append_to_textbuffer(GTK_TEXT_VIEW(glade_xml_get_widget(xml, "details_textview")), waitstring->str);
			gdk_flush();
			gdk_threads_leave();

			g_timer_start(timer);

			packages = guyum_main_execute_query(guyummain, query, searchtext, searchfields, repoid);
			
			if (packages!=NULL) {

				g_timer_stop(timer);
				g_string_printf(waitstring, _("%d packages found.\nTime elapsed: %d sec\n"), guyum_package_composite_get_component_count(packages), (int)g_timer_elapsed(timer, &microseconds));
				
				gdk_threads_enter();
	
				append_to_textbuffer(GTK_TEXT_VIEW(glade_xml_get_widget(xml, "details_textview")), waitstring->str);
				gdk_flush();
				gdk_threads_leave();			
	
				g_thread_yield();
				g_string_free(waitstring, TRUE);
			} else {
				gdk_threads_enter();
				show_error_dialog("An unexpected error occured.");
				gdk_flush();
				gdk_threads_leave();			
	
				g_thread_yield();
				
				cancelsearch=1;
			}
		} 
	}

	g_async_queue_unref(asyncqueue);	

	g_idle_add(search_notify, query);
	
	if (timer!=NULL) {
		g_timer_destroy(timer);
	}	

	/* Free variables */
	g_list_free(repositories);	
	g_list_free(searchfields);		
	
	return FALSE;
}
