/***************************************************************************
 *            t_install.c
 *
 *  Sun Feb 11 14:04:05 2007
 *  Copyright  2007  User
 *  Email
 ****************************************************************************/
 
#include "test_all.h"
#include "test_utils.h"
#include <include/init.h>

static gchar* REPOID="localtest";
static gint globalcounter = 0;
extern GuyumMain* guyummain;

#define EXECINSTALL 0

/**
* Test for simple installation 
* TI10
*/
void TestSearch_SimpleInstall(CuTest* t)
{
	GList* namelist = NULL;
	GuyumPackageComposite* packages = NULL;
	GuyumPackageComponent* package = NULL;	
	GuyumPackageComposite* deps = NULL;
	gchar* pkgid;
	GList* repositories = NULL;
	gchar* totalamount = NULL;
	
	GValue strvalue={0,};
	GValue ptrvalue={0,};
	g_value_init(&strvalue, G_TYPE_STRING);
	g_value_init(&ptrvalue, G_TYPE_POINTER);
	
	DEBUG_OUTPUT("Running simple installation tests...");
	DEBUG_OUTPUT("TI10");	
	
	namelist = g_list_append(namelist, "name");
	repositories = g_list_append(repositories, "REPOID");
	
	/* Search for install package */
	packages = runSearch("guyum-install", namelist, globalcounter, REPOID);
	CuAssertPtrNotNull(t, packages);
	
	package = guyum_package_composite_get_nth_component(packages, 0);	
	CuAssertPtrNotNull(t, package);
	
	g_object_get_property(G_OBJECT(package), "packageid", &strvalue);
	CuAssertPtrNotNull(t, &strvalue);
	
	pkgid = g_value_get_string(&strvalue);
	
	/* Search for dependencies */
	deps = guyum_main_finddeps(guyummain, globalcounter, pkgid);
	CuAssertPtrNotNull(t, deps);
	/* there are no dependencies for this package */
	CuAssertIntEquals(t, 1, guyum_package_composite_get_component_count(deps));
	
	/* Assertion: install component exists now */
	g_object_get_property(G_OBJECT(guyummain), "install", &ptrvalue);
	CuAssertPtrNotNull(t, &ptrvalue);
	
	/* Check for download size*/
	totalamount = guyum_main_get_totalamount(guyummain, 0);

/* Since createrepo 0.4: 2 kB*/
#if 0
	CuAssertStrEquals(t, "1 kB", totalamount);
#endif
	/* Execute install */
#if EXECINSTALL	
	guyum_main_execute_install(guyummain, globalcounter);
#endif

	globalcounter++;

	g_value_unset(&strvalue);
	g_value_unset(&ptrvalue);
	g_list_free(namelist);
	g_list_free(repositories);
	
}


/**
* Test for dependency installation 
* TI20
*/
void TestSearch_DepInstall(CuTest* t)
{
	GList* namelist = NULL;
	GuyumPackageComposite* packages = NULL;
	GuyumPackageComponent* package = NULL;	
	GuyumPackageComposite* deps = NULL;
	gchar* pkgid;
	GList* repositories = NULL;
	gchar* totalamount = NULL;
	
	GValue strvalue={0,};
	GValue ptrvalue={0,};
	g_value_init(&strvalue, G_TYPE_STRING);
	g_value_init(&ptrvalue, G_TYPE_POINTER);
	
	DEBUG_OUTPUT("Running dependency installation tests...");
	DEBUG_OUTPUT("TI20");	
	
	namelist = g_list_append(namelist, "name");
	repositories = g_list_append(repositories, "REPOID");
	
	/* Search for install package */
	packages = runSearch("guyum-depinst", namelist, globalcounter, REPOID);
	CuAssertPtrNotNull(t, packages);
	
	package = guyum_package_composite_get_nth_component(packages, 0);	
	CuAssertPtrNotNull(t, package);
	
	g_object_get_property(G_OBJECT(package), "packageid", &strvalue);
	CuAssertPtrNotNull(t, &strvalue);
	
	pkgid = g_value_get_string(&strvalue);
	/* Search for dependencies */
	deps = guyum_main_finddeps(guyummain, globalcounter, pkgid);
	CuAssertPtrNotNull(t, deps);
	/* there are no dependencies for this package */
	CuAssertIntEquals(t, 3, guyum_package_composite_get_component_count(deps));
	
	/* Assertion: install component exists now */
	g_object_get_property(G_OBJECT(guyummain), "install", &ptrvalue);
	CuAssertPtrNotNull(t, &ptrvalue);
	
	/* Check for download size*/
	totalamount = guyum_main_get_totalamount(guyummain, 0);

	CuAssertStrEquals(t, "6 kB", totalamount);
	
	
	/* Execute install */
#if EXECINSTALL	
	guyum_main_execute_install(guyummain, globalcounter);
#endif

	globalcounter++;

	g_value_unset(&strvalue);
	g_value_unset(&ptrvalue);
	g_list_free(namelist);
	g_list_free(repositories);
	
}


CuSuite* make_installtest_suite(void)
{
	CuSuite *s=CuSuiteNew();	
	
	SUITE_ADD_TEST(s, TestSearch_SimpleInstall);
	SUITE_ADD_TEST(s, TestSearch_DepInstall);
	
	return s;
}
