/***************************************************************************
 *            t_package.c
 *
 *  Tue Feb  6 22:20:21 2007
 *  Copyright  2007  User
 *  Email
 ****************************************************************************/
#include "test_all.h"
#include <include/init.h>
#include "test_utils.h"

static gchar* REPOID="localtest";
extern GuyumMain* guyummain;

#define ERROR 0
#define YUMVERSION 300
/**
* Test for remote package
* TP20
*/
void TestSearch_PackRemoteAll(CuTest* t)
{
	GList* namelist = NULL;
	GuyumPackageComposite* packages = NULL;
	GuyumPackageComponent* package = NULL;	
	
	GValue strvalue={0,};
	GValue ptrvalue={0,};
	g_value_init(&strvalue, G_TYPE_STRING);
	g_value_init(&ptrvalue, G_TYPE_POINTER);
	
	DEBUG_OUTPUT("Running remote package properties tests...");
	DEBUG_OUTPUT("TP20");	
	
	namelist = g_list_append(namelist, "name");
	
	/* Search for existing package */
	packages = runSearch("guyum-package-remote", namelist, 0, REPOID);

	CuAssertPtrNotNull(t, packages);
	
	package = guyum_package_composite_get_nth_component(packages, 0);
	
	CuAssertPtrNotNull(t, package);
	
	g_object_get_property(G_OBJECT(package), "arch", &strvalue);	
	CuAssertStrEquals(t, "noarch", g_value_get_string(&strvalue) );	
	
	/* For remote packages changelog is not available before yum 3.0.1 */
	g_object_get_property(G_OBJECT(package), "changelog", &strvalue);	
	CuAssertStrEquals(t, "", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "compsize", &strvalue);	

	CuAssertStrEquals(t, "2 kB", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "conflicts", &ptrvalue);	
	CuAssertIntEquals(t, 1, g_list_length((GList*)g_value_get_pointer(&ptrvalue)) );	
	
	g_object_get_property(G_OBJECT(package), "description", &strvalue);	
	CuAssertStrEquals(t, "guyum-package-remote - Package for testing guyum package functionality, remote package.", g_value_get_string(&strvalue) );

	g_object_get_property(G_OBJECT(package), "epoch", &strvalue);	
	CuAssertStrEquals(t, "0", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "files", &ptrvalue);	
	CuAssertIntEquals(t, 0, g_list_length((GList*)g_value_get_pointer(&ptrvalue)) );	

	g_object_get_property(G_OBJECT(package), "group", &strvalue);	
	CuAssertStrEquals(t, "Development/Tools", g_value_get_string(&strvalue) );	

#if ERROR
	g_object_get_property(G_OBJECT(package), "licenses", &strvalue);	
	CuAssertStrEquals(t, "GPL", g_value_get_string(&strvalue) );	
#endif

	g_object_get_property(G_OBJECT(package), "localrelease", &strvalue);	
	CuAssertStrEquals(t, "", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "localversion", &strvalue);	
	CuAssertStrEquals(t, "", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "name", &strvalue);	
	CuAssertStrEquals(t, "guyum-package-remote", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "obsoletes", &ptrvalue);	
	CuAssertIntEquals(t, 1, g_list_length((GList*)g_value_get_pointer(&ptrvalue)) );	

	/*g_object_get_property(G_OBJECT(package), "packageid", &strvalue);	
	CuAssertStrEquals(t, "21cabfe04a9e0d69db19660b6932885a231795f0", g_value_get_string(&strvalue) );	*/

	g_object_get_property(G_OBJECT(package), "pkgsize", &strvalue);	

	CuAssertStrEquals(t, "2 kB", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "provides", &ptrvalue);	
	CuAssertIntEquals(t, 1, g_list_length((GList*)g_value_get_pointer(&ptrvalue)) );	

	g_object_get_property(G_OBJECT(package), "release", &strvalue);	
	CuAssertStrEquals(t, "1", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "remoterelease", &strvalue);	
	CuAssertStrEquals(t, "1", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "remoteversion", &strvalue);	
	CuAssertStrEquals(t, "0.1.0", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "repoid", &strvalue);	
	CuAssertStrEquals(t, "localtest", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "requires", &ptrvalue);	
	CuAssertIntEquals(t, 1, g_list_length((GList*)g_value_get_pointer(&ptrvalue)) );	

	g_object_get_property(G_OBJECT(package), "state", &strvalue);	
	CuAssertStrEquals(t, "i", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "summary", &strvalue);	
	CuAssertStrEquals(t, "guyum package functionality test", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "url", &strvalue);	
	CuAssertStrEquals(t, "http://www.guyum.de/", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "vendor", &strvalue);	
	CuAssertStrEquals(t, "GuYum project", g_value_get_string(&strvalue) );

	g_object_get_property(G_OBJECT(package), "version", &strvalue);	
	CuAssertStrEquals(t, "0.1.0", g_value_get_string(&strvalue) );	

	DEBUG_OUTPUT("Done.\n");

	g_value_unset(&strvalue);
	g_value_unset(&ptrvalue);
	
	g_list_free(namelist);
	
}


/**
* Test for remote and local package
* TP10
*/
void TestSearch_PackRemoteLocalAll(CuTest* t)
{
	GList* namelist = NULL;
	GuyumPackageComposite* packages = NULL;
	GuyumPackageComponent* package = NULL;	
	
	GValue strvalue={0,};
	GValue ptrvalue={0,};
	g_value_init(&strvalue, G_TYPE_STRING);
	g_value_init(&ptrvalue, G_TYPE_POINTER);
	
	DEBUG_OUTPUT("Running remote/local package properties tests...");
	DEBUG_OUTPUT("TP10");	
	
	namelist = g_list_append(namelist, "name");
	
	/* Search for existing package */
	packages = runSearch("guyum-package-loc-rem", namelist, 0, REPOID);
	
	CuAssertPtrNotNull(t, packages);	
	
	package = guyum_package_composite_get_nth_component(packages, 0);
	
	CuAssertPtrNotNull(t, package);
	
	g_object_get_property(G_OBJECT(package), "arch", &strvalue);	
	CuAssertStrEquals(t, "noarch", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "changelog", &strvalue);	
	CuAssertStrEquals(t, "2007-02-08 GuYum project 0.1.0-1 \n- Initial RPM", g_value_get_string(&strvalue) );	
	
	g_object_get_property(G_OBJECT(package), "compsize", &strvalue);	
	CuAssertStrEquals(t, "1 Bytes", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "conflicts", &ptrvalue);	
	CuAssertIntEquals(t, 1, g_list_length((GList*)g_value_get_pointer(&ptrvalue)) );

	g_object_get_property(G_OBJECT(package), "description", &strvalue);	
	CuAssertStrEquals(t, "guyum-package-loc-rem - Package for testing guyum package functionality, local and remote package.\n", g_value_get_string(&strvalue) );

	g_object_get_property(G_OBJECT(package), "epoch", &strvalue);	
	CuAssertStrEquals(t, "0", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "files", &ptrvalue);	
	CuAssertIntEquals(t, 1, g_list_length((GList*)g_value_get_pointer(&ptrvalue)) );	

	g_object_get_property(G_OBJECT(package), "group", &strvalue);	
	CuAssertStrEquals(t, "Development/Tools", g_value_get_string(&strvalue) );	

#if ERROR
	g_object_get_property(G_OBJECT(package), "licenses", &strvalue);	
	CuAssertStrEquals(t, "GPL", g_value_get_string(&strvalue) );
#endif

	g_object_get_property(G_OBJECT(package), "localrelease", &strvalue);	
	CuAssertStrEquals(t, "1", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "localversion", &strvalue);	
	CuAssertStrEquals(t, "0.1.0", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "name", &strvalue);	
	CuAssertStrEquals(t, "guyum-package-loc-rem", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "obsoletes", &ptrvalue);	
	CuAssertIntEquals(t, 1, g_list_length((GList*)g_value_get_pointer(&ptrvalue)) );

	/*g_object_get_property(G_OBJECT(package), "packageid", &strvalue);	
	CuAssertStrEquals(t, "a78ee58cf210d1169ecb07bde356001cc6aaecfe", g_value_get_string(&strvalue) );	*/

	g_object_get_property(G_OBJECT(package), "pkgsize", &strvalue);	
	CuAssertStrEquals(t, "1 Bytes", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "provides", &ptrvalue);	
	CuAssertIntEquals(t, 1, g_list_length((GList*)g_value_get_pointer(&ptrvalue)) );

	g_object_get_property(G_OBJECT(package), "release", &strvalue);	
	CuAssertStrEquals(t, "1", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "remoterelease", &strvalue);	
	CuAssertStrEquals(t, "1", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "remoteversion", &strvalue);	
	CuAssertStrEquals(t, "0.1.0", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "repoid", &strvalue);	
	CuAssertStrEquals(t, "localtest", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "requires", &ptrvalue);	
	CuAssertIntEquals(t, 1, g_list_length((GList*)g_value_get_pointer(&ptrvalue)) );	

	g_object_get_property(G_OBJECT(package), "state", &strvalue);	
	CuAssertStrEquals(t, "o", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "summary", &strvalue);	
	CuAssertStrEquals(t, "guyum package functionality test", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "url", &strvalue);	
	CuAssertStrEquals(t, "http://www.guyum.de/", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "vendor", &strvalue);	
	CuAssertStrEquals(t, "GuYum project", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "version", &strvalue);	
	CuAssertStrEquals(t, "0.1.0", g_value_get_string(&strvalue) );	

	DEBUG_OUTPUT("Done.\n");

	g_value_unset(&strvalue);
	g_value_unset(&ptrvalue);
	
	g_list_free(namelist);
	
}


/**
* Test for local package
* TP30
*/
void TestSearch_PackLocalAll(CuTest* t)
{
	GList* namelist = NULL;
	GuyumPackageComposite* packages = NULL;
	GuyumPackageComponent* package = NULL;	
	
	GValue strvalue={0,};
	GValue ptrvalue={0,};
	g_value_init(&strvalue, G_TYPE_STRING);
	g_value_init(&ptrvalue, G_TYPE_POINTER);
	
	DEBUG_OUTPUT("Running local package properties tests...");
	DEBUG_OUTPUT("TP30");	
	
	namelist = g_list_append(namelist, "name");
	
	/* Search for existing package */
	packages = runSearch("guyum-package-local", namelist, 0, REPOID);

	CuAssertPtrNotNull(t, packages);
	
	package = guyum_package_composite_get_nth_component(packages, 0);
	
	CuAssertPtrNotNull(t, package);
	
	g_object_get_property(G_OBJECT(package), "arch", &strvalue);	
	CuAssertStrEquals(t, "noarch", g_value_get_string(&strvalue) );	

#if ERROR	
	g_object_get_property(G_OBJECT(package), "changelog", &strvalue);	
	CuAssertStrEquals(t, "", g_value_get_string(&strvalue) );	
#endif
	
	g_object_get_property(G_OBJECT(package), "compsize", &strvalue);	
	CuAssertStrEquals(t, "1 Bytes", g_value_get_string(&strvalue) );	

#if ERROR
	g_object_get_property(G_OBJECT(package), "conflicts", &ptrvalue);	
	CuAssertIntEquals(t, 1, g_list_length((GList*)g_value_get_pointer(&ptrvalue)) );
#endif

	g_object_get_property(G_OBJECT(package), "description", &strvalue);	
	CuAssertStrEquals(t, "guyum-package-local - Package for testing guyum package functionality, local package.\n", g_value_get_string(&strvalue) );

	g_object_get_property(G_OBJECT(package), "epoch", &strvalue);	
	CuAssertStrEquals(t, "0", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "files", &ptrvalue);	
	CuAssertIntEquals(t, 1, g_list_length((GList*)g_value_get_pointer(&ptrvalue)) );	

	g_object_get_property(G_OBJECT(package), "group", &strvalue);	
	CuAssertStrEquals(t, "Development/Tools", g_value_get_string(&strvalue) );	

#if ERROR
	g_object_get_property(G_OBJECT(package), "licenses", &strvalue);	
	CuAssertStrEquals(t, "GPL", g_value_get_string(&strvalue) );
#endif

	g_object_get_property(G_OBJECT(package), "localrelease", &strvalue);	
	CuAssertStrEquals(t, "1", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "localversion", &strvalue);	
	CuAssertStrEquals(t, "0.1.0", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "name", &strvalue);	
	CuAssertStrEquals(t, "guyum-package-local", g_value_get_string(&strvalue) );	

#if ERROR 
	g_object_get_property(G_OBJECT(package), "obsoletes", &ptrvalue);	
	CuAssertIntEquals(t, 1, g_list_length((GList*)g_value_get_pointer(&ptrvalue)) );
#endif

	/*g_object_get_property(G_OBJECT(package), "packageid", &strvalue);	
	CuAssertStrEquals(t, "1e765d9125f365239ed1a30bd0a4c60cb1488413", g_value_get_string(&strvalue) );	*/

	g_object_get_property(G_OBJECT(package), "pkgsize", &strvalue);	
	CuAssertStrEquals(t, "1 Bytes", g_value_get_string(&strvalue) );	

#if ERROR
	g_object_get_property(G_OBJECT(package), "provides", &ptrvalue);	
	CuAssertIntEquals(t, 1, g_list_length((GList*)g_value_get_pointer(&ptrvalue)) );
#endif

	g_object_get_property(G_OBJECT(package), "release", &strvalue);	
	CuAssertStrEquals(t, "1", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "remoterelease", &strvalue);	
	CuAssertStrEquals(t, "", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "remoteversion", &strvalue);	
	CuAssertStrEquals(t, "", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "repoid", &strvalue);	
	CuAssertStrEquals(t, "installed", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "requires", &ptrvalue);	
	CuAssertIntEquals(t, 1, g_list_length((GList*)g_value_get_pointer(&ptrvalue)) );	

	g_object_get_property(G_OBJECT(package), "state", &strvalue);	
	CuAssertStrEquals(t, "o", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "summary", &strvalue);	
	CuAssertStrEquals(t, "guyum package functionality test", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "url", &strvalue);	
	CuAssertStrEquals(t, "http://www.guyum.de/", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "vendor", &strvalue);	
	CuAssertStrEquals(t, "GuYum project", g_value_get_string(&strvalue) );	

	g_object_get_property(G_OBJECT(package), "version", &strvalue);	
	CuAssertStrEquals(t, "0.1.0", g_value_get_string(&strvalue) );	

	DEBUG_OUTPUT("Done.\n");

	g_value_unset(&strvalue);
	g_value_unset(&ptrvalue);
	
	g_list_free(namelist);
	
}

CuSuite* make_packagetest_suite(void)
{
	CuSuite *s=CuSuiteNew();	
	
	SUITE_ADD_TEST(s, TestSearch_PackRemoteLocalAll);
	SUITE_ADD_TEST(s, TestSearch_PackRemoteAll);
	SUITE_ADD_TEST(s, TestSearch_PackLocalAll);
	
	return s;
}
