/***************************************************************************
 *            t_searchtest.c
 *
 *  Mon Feb  5 13:12:54 2007
 *  Copyright  2007  User
 *  Email
 ****************************************************************************/
 
#include "test_all.h"
#include "test_utils.h"
#include <include/init.h>

static gchar* REPOID="localtest";
static gint globalcounter = 0;
extern GuyumMain* guyummain;

/**
* 
*/
void 
testsearch_execquery(CuTest* t, GList* searchfields, gchar* repo)
{
	g_return_if_fail(searchfields!=NULL);
	GuyumPackageComposite* packages = NULL;
		
	/* Search for not existing string */
	packages = runSearch("#p50sv+s2059", searchfields, globalcounter++, repo);	
	CuAssertIntEquals(t, 0, guyum_package_composite_get_component_count(packages));
	
	/* Search for guyum (assume it's in localtest) */
	packages = runSearch("guyum-search", searchfields, globalcounter++, repo);	
	CuAssertIntEquals(t, 2, guyum_package_composite_get_component_count(packages));	
			
}

/**
* Run simple queries
* TS10, TS11, TS12, TS20, TS21, TS22
*/
void TestSearch_SimpleQuery(CuTest* t)
{
	GList* namelist = NULL;
	GList* desclist = NULL;
	GList* alllist = NULL;
	
	DEBUG_OUTPUT("Running simple query tests...");
	
	DEBUG_OUTPUT("TS10 TS20");
	namelist = g_list_append(namelist, "name");
	testsearch_execquery(t, namelist, REPOID);
	
	DEBUG_OUTPUT("TS11 TS21");
	desclist = g_list_append(desclist, "description");
	testsearch_execquery(t, desclist, REPOID);
	
	DEBUG_OUTPUT("TS12 TS22");
	alllist = g_list_append(alllist, "name");
	alllist = g_list_append(alllist, "description");
	testsearch_execquery(t, alllist, REPOID);
	
	DEBUG_OUTPUT("Done.\n");
	
	
	g_list_free(namelist);
	g_list_free(desclist);
	g_list_free(alllist);
}

/**
* Repository list is empty
* TS30
*/
void TestSearch_EmptyRepo(CuTest* t)
{
	GList* namelist = NULL;
	GuyumPackageComposite* packages = NULL;
			
	DEBUG_OUTPUT("Searching in NULL-repository");
	
	DEBUG_OUTPUT("TS30");
	namelist = g_list_append(namelist, "name");
		
	/* Search for not existing string */
	packages = runSearch("#p50sv+s2059", namelist, globalcounter++, NULL);	
	CuAssertPtrEquals(t, packages, NULL);	
	
	DEBUG_OUTPUT("Done.\n");	
	
	g_list_free(namelist);
}


/**
* Search in a real repository
* TS40, TS41, TS42
*/
void TestSearch_RealRepo(CuTest* t)
{
	GList* namelist = NULL;
	GuyumPackageComposite* packages = NULL;
			
	DEBUG_OUTPUT("Searching in real repository");
	
	DEBUG_OUTPUT("TS40");
	namelist = g_list_append(namelist, "name");
		
	/* Search for not existing string */
	packages = runSearch("#p50sv+s2059", namelist, globalcounter++, "base");
	CuAssertIntEquals(t, 0, guyum_package_composite_get_component_count(packages));

	DEBUG_OUTPUT("TS41");
	/* Search for not existing string */
	packages = runSearch("perl", namelist, globalcounter++, "base");	
	CuAssertPtrNotNull(t, packages);	
	
	DEBUG_OUTPUT("Done.\n");	
	
	g_list_free(namelist);
}


CuSuite* make_searchtest_suite(void)
{
	CuSuite *s=CuSuiteNew();	
	
	SUITE_ADD_TEST(s, TestSearch_SimpleQuery);
	SUITE_ADD_TEST(s, TestSearch_EmptyRepo);
	SUITE_ADD_TEST(s, TestSearch_RealRepo);	
	
	return s;
}
