#include <include/ui/ui_deps.h>

extern GuyumMain* guyummain;
extern GladeXML* xml;

/**
* Liest Informationen zu dem Package aus und stellt sie dar
*/
void
set_deps_package_information(const gchar* packageid, GuyumPackageComposite* packages)
{
	GuyumPackageComponent* package;
	
	package = guyum_package_composite_get_id_component(packages, packageid);
	
	if (package!=NULL) {
		gchar* group;
		gchar* summary;
		gchar* url;
		gchar* vendor;
		gchar* licenses;
		gchar* description;
		GList* obsoletes = NULL;
		GList* requires = NULL;
		GList* provides = NULL;
		GList* conflicts = NULL;
				
		GOBJECT_GET_POINTER_PROPERTY(package, "conflicts", conflicts);
		if (conflicts==NULL) {
			conflicts = g_list_append(conflicts, "");
		} 
		write_list_into_tree(conflicts, GTK_TREE_VIEW(glade_xml_get_widget(xml, "deps_treeview_conflicts")), "conflicts");			
		
		GOBJECT_GET_STRING_PROPERTY(package, "description", &description);
		if (description!=NULL) {
			write_into_textbuffer(GTK_TEXT_VIEW(glade_xml_get_widget(xml, "deps_textview_description")), description);
			g_free(description);
		}

		
		GOBJECT_GET_STRING_PROPERTY(package, "group", &group);
		if (group!=NULL) {
			gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "deps_entry_category")), group);
			g_free(group);
		}

		GOBJECT_GET_STRING_PROPERTY(package, "licenses", &licenses);
		if (licenses!=NULL) {
			gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "deps_entry_licenses")), licenses);
			g_free(licenses);
		}

		GOBJECT_GET_POINTER_PROPERTY(package, "obsoletes", obsoletes);
		if (obsoletes==NULL) {
			obsoletes = g_list_append(obsoletes, "");
		}
		write_list_into_tree(obsoletes, GTK_TREE_VIEW(glade_xml_get_widget(xml, "deps_treeview_obsoletes")), "obsoletes");
		
		
		GOBJECT_GET_POINTER_PROPERTY(package, "provides", provides);
		if (provides==NULL) {
			provides = g_list_append(provides, "");
		} 
		write_list_into_tree(provides, GTK_TREE_VIEW(glade_xml_get_widget(xml, "deps_treeview_provides")), "provides");			
		
		
		GOBJECT_GET_POINTER_PROPERTY(package, "requires", requires);
		if (requires==NULL) {			
			requires = g_list_append(requires, "");
		} 
		write_list_into_tree(requires, GTK_TREE_VIEW(glade_xml_get_widget(xml, "deps_treeview_requires")), "requires");			
		
		GOBJECT_GET_STRING_PROPERTY(package, "summary", &summary);
		if (summary!=NULL) {
			write_into_textbuffer(GTK_TEXT_VIEW(glade_xml_get_widget(xml, "deps_textview_summary")), summary);
			g_free(summary);
		}

		GOBJECT_GET_STRING_PROPERTY(package, "url", &url);
		if (url!=NULL) {
			gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "deps_entry_url")), url);
			g_free(url);
		}

		GOBJECT_GET_STRING_PROPERTY(package, "vendor", &vendor);		
		if (vendor!=NULL) {
			gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "deps_entry_vendor")), vendor);
			g_free(vendor);
		}

	} else {
		show_error_dialog("Error: Package information not available");
	}
	
}



/**
*
*/
void
deps_selection_changed (GtkTreeSelection *selection, gpointer user_data)
{
  	GtkTreeModel     *model;
	GtkTreeIter      iter;
	gchar* pkgid = NULL;
	GuyumPackageComposite* packages = (GuyumPackageComposite*)user_data;

	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_tree_model_get (model, &iter, COL_DEPPACKAGE_PKGID, &pkgid, -1);
		set_deps_package_information(pkgid, packages);	
	}
	/* Free variables */
	g_free(pkgid);	
	
}
