/***************************************************************************
 *            deps_tree.c
 *
 *  Sat Jul  1 22:08:40 2006
 *  Copyright  2006  User
 *  Email
 ****************************************************************************/

#include <include/ui/ui_deps_tree.h>

extern GuyumMain* guyummain;
extern GladeXML* xml;
extern GtkWidget* window1;
extern GtkWidget* logdialog;
extern GdkCursor* cursor;
extern GtkWidget* depsdialog;

/**
*
*/
GtkTreeView*
create_deppackages_treeview()
{
	GtkTreeViewColumn 	*col;
	GtkCellRenderer 	*renderer;
	GtkTreeStore *treestore;
	GtkTreeView* treeview;
	
	treestore = gtk_tree_store_new(NUM_DEPPACKAGE_COLS, 
										G_TYPE_STRING, 
										G_TYPE_STRING, 
										G_TYPE_STRING, 
										G_TYPE_STRING, 
										G_TYPE_STRING,
										G_TYPE_STRING);	
	
	treeview = GTK_TREE_VIEW(gtk_tree_view_new_with_model(GTK_TREE_MODEL(treestore)));
	
	/* Set rules hint */
	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(treeview), TRUE);
			
	/* Column name */
	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(col, "Name");
	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), col);
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col, renderer, "text", COL_DEPPACKAGE_NAME);		

	/* Column remote version */
	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(col, "Remote");
	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), col);
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col, renderer, "text", COL_DEPPACKAGE_REMOTEVERSION);	

	/* Column size */
	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(col, "Download size");
	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), col);
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col, renderer, "text", COL_DEPPACKAGE_SIZE);
	
	/* Column repoid */
	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(col, "Repository");
	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), col);
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col, renderer, "text", COL_DEPPACKAGE_REPOID);	
	
	/* Column pkgid: hidden */
	col = gtk_tree_view_column_new();
	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), col);                         
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col, renderer, "text", COL_DEPPACKAGE_PKGID);	
	gtk_tree_view_column_set_visible(col, FALSE);	

	/* Column description: hidden */
	col = gtk_tree_view_column_new();
	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), col);                         
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col, renderer, "text", COL_DEPPACKAGE_DESCRIPTION);	
	gtk_tree_view_column_set_visible(col, FALSE);	
	
	return GTK_TREE_VIEW(treeview);
	
}


/**
*
*/
void
deppackages_create_and_fill_model(GuyumPackageComposite* packages, GtkTreeView* treeview)
{
	GtkTreeStore *treestore;
	GtkTreeIter toplevel;
	gint i = 0;
	GtkTreeModel *sortmodel;
	
	/* packages treestore */
	treestore = GTK_TREE_STORE(gtk_tree_view_get_model(treeview));

	for (i=0; i<guyum_package_composite_get_component_count(packages); i++) {		
		gchar* namevalue;
		gchar* remoteversionvalue ;
		gchar* pkgidvalue;
		gchar* descvalue ;
		gchar* sizevalue;
		gchar* repoidvalue;
		
		GuyumPackageComponent* package = guyum_package_composite_get_nth_component(packages, i);
						
		GOBJECT_GET_STRING_PROPERTY(package, "name", &namevalue);
		GOBJECT_GET_STRING_PROPERTY(package, "remoteversion", &remoteversionvalue);
		GOBJECT_GET_STRING_PROPERTY(package, "pkgsize", &sizevalue);
		GOBJECT_GET_STRING_PROPERTY(package, "repoid", &repoidvalue);
		GOBJECT_GET_STRING_PROPERTY(package, "packageid", &pkgidvalue);
		GOBJECT_GET_STRING_PROPERTY(package, "description", &descvalue);
		
		/* append new line to treeview */
        gtk_tree_store_append(treestore, &toplevel, NULL);
		gtk_tree_store_set(treestore, &toplevel, 
							COL_DEPPACKAGE_NAME, namevalue,
							COL_DEPPACKAGE_REMOTEVERSION, remoteversionvalue,
							COL_DEPPACKAGE_SIZE, sizevalue,
							COL_DEPPACKAGE_REPOID, repoidvalue,
							COL_DEPPACKAGE_PKGID, pkgidvalue,
							COL_DEPPACKAGE_DESCRIPTION, descvalue,
							-1);

		g_free(namevalue);
		g_free(remoteversionvalue);
		g_free(pkgidvalue);
		g_free(descvalue);
		g_free(sizevalue);
		g_free(repoidvalue);
		
	}
	sortmodel = gtk_tree_model_sort_new_with_model(GTK_TREE_MODEL(treestore));
	gtk_tree_view_set_model(GTK_TREE_VIEW((treeview)), GTK_TREE_MODEL(sortmodel));
	
	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(sortmodel), COL_PACKAGE_NAME, GTK_SORT_ASCENDING);
	
	g_object_unref(treestore);
}
