
#include <include/ui/ui_query.h>

extern GuyumMain* guyummain;
extern GladeXML* xml;


/**
* Liest Informationen zu dem Package aus und stellt sie dar
*/
void
set_package_information(const gchar* packageid, GuyumPackageComposite* packages)
{
	GuyumPackageComponent* package;
	
	package = guyum_package_composite_get_id_component(packages, packageid);
	
	
	if (package!=NULL) {
		GValue strvalue={0,};
		GValue ptrvalue={0,};
		
		g_value_init(&strvalue, G_TYPE_STRING);
		g_value_init(&ptrvalue, G_TYPE_POINTER);		

		g_object_get_property(G_OBJECT(package), "changelog", &strvalue);
		write_into_textbuffer(GTK_TEXT_VIEW(glade_xml_get_widget(xml, "guyum_textview_changelog")), g_value_get_string(&strvalue));
				
		g_object_get_property(G_OBJECT(package), "conflicts", &ptrvalue);
		write_list_into_tree((GList*)(g_value_get_pointer(&ptrvalue)), GTK_TREE_VIEW(glade_xml_get_widget(xml, "guyum_treeview_conflicts")), "conflicts");			
		
		g_object_get_property(G_OBJECT(package), "description", &strvalue);
		write_into_textbuffer(GTK_TEXT_VIEW(glade_xml_get_widget(xml, "guyum_textview_package_description")), g_value_get_string(&strvalue));

		g_object_get_property(G_OBJECT(package), "files", &ptrvalue);
		write_list_into_tree((GList*)(g_value_get_pointer(&ptrvalue)), GTK_TREE_VIEW(glade_xml_get_widget(xml, "guyum_treeview_files")), "files");			

		g_object_get_property(G_OBJECT(package), "group", &strvalue);
		gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "guyum_entry_category")), g_value_get_string(&strvalue));

		g_object_get_property(G_OBJECT(package), "licenses", &strvalue);
		gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "guyum_entry_licenses")), g_value_get_string(&strvalue));

		g_object_get_property(G_OBJECT(package), "obsoletes", &ptrvalue);		
		write_list_into_tree((GList*)(g_value_get_pointer(&ptrvalue)), GTK_TREE_VIEW(glade_xml_get_widget(xml, "guyum_treeview_obsoletes")), "obsoletes");		
		
		g_object_get_property(G_OBJECT(package), "provides", &ptrvalue);		
		write_list_into_tree((GList*)(g_value_get_pointer(&ptrvalue)), GTK_TREE_VIEW(glade_xml_get_widget(xml, "guyum_treeview_provides")), "provides");			
					
		g_object_get_property(G_OBJECT(package), "requires", &ptrvalue);		
		write_list_into_tree((GList*)(g_value_get_pointer(&ptrvalue)), GTK_TREE_VIEW(glade_xml_get_widget(xml, "guyum_treeview_requires")), "requires");					
		
		g_object_get_property(G_OBJECT(package), "summary", &strvalue);
		write_into_textbuffer(GTK_TEXT_VIEW(glade_xml_get_widget(xml, "guyum_textview_summary")), g_value_get_string(&strvalue));

		g_object_get_property(G_OBJECT(package), "url", &strvalue);
		gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "guyum_entry_url")), g_value_get_string(&strvalue));

		g_object_get_property(G_OBJECT(package), "vendor", &strvalue);
		gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "guyum_entry_vendor")), g_value_get_string(&strvalue));
		
		g_value_unset(&strvalue);
		g_value_unset(&ptrvalue);

	} else {
		show_error_dialog("Error: Package information not available");
	}
	
	
	
}


/**
*
*/
GtkTreeView* get_nth_query_treeview(GtkNotebook* notebook_query, guint nth)
{
	
	GtkWidget* scrolledwindow;
	GtkTreeView *treeview;	
	GList* children;
	
	notebook_query = GTK_NOTEBOOK(glade_xml_get_widget(xml, "guyum_notebook_query"));	
	scrolledwindow = GTK_WIDGET(gtk_notebook_get_nth_page(notebook_query, nth));
	/* */
	children = gtk_container_get_children(GTK_CONTAINER(scrolledwindow));
	treeview = GTK_TREE_VIEW(g_list_nth_data(children, 0));
	
	return treeview;
}


/**
* create new tabulator label
*/
GtkWidget* create_tablabel(gchar* labeltext, gpointer* query)
{
	GtkWidget* tablabel;
	GtkWidget* tabclosebutton;
	GtkWidget* image;
	GtkWidget* label;
	/* is a hbox containing a button and a label */
	tablabel = gtk_hbox_new(FALSE, 1);
	/* close button */
	tabclosebutton = gtk_button_new();
	gtk_button_set_relief(GTK_BUTTON(tabclosebutton), GTK_RELIEF_NONE);
	image = gtk_image_new_from_stock(GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
	gtk_widget_set_size_request(image, 9, 9);
	gtk_container_add(GTK_CONTAINER(tabclosebutton), image);
	/* label */
	label = gtk_label_new(labeltext);
	/* pack it together */
	gtk_box_pack_start(GTK_BOX(tablabel), label, TRUE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(tablabel), tabclosebutton, FALSE, FALSE, 0);
	gtk_widget_show_all(tablabel);
	/* on close */
	g_signal_connect(tabclosebutton, "clicked", G_CALLBACK(on_guyum_button_close_current_clicked), query);
	
	return tablabel;
}



/**
* Add packages to the notebook
*/
void
add_packages_tab(GuyumQueryComponent* query, gint position)
{
	gchar* searchtext;	
	gchar* searchstring;

	GtkTreeSelection* selection;
	GtkTreeIter treepath;
	GtkTreeModel* treemodel;
	GtkTreeView* treeview;
	GtkWidget* child;
	GtkNotebook* notebook_query;		
	gint currentpage = 0;
	
	GuyumPackageComposite* allpackages;
		
	GOBJECT_GET_POINTER_PROPERTY(query, "packages", allpackages);
	
	notebook_query = GTK_NOTEBOOK(glade_xml_get_widget(xml, "guyum_notebook_query"));	

	child = gtk_scrolled_window_new(NULL, NULL);	
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(child), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	treeview = create_query_treeview();
	
	gtk_widget_show(GTK_WIDGET(treeview));
	gtk_container_add(GTK_CONTAINER(child), GTK_WIDGET(treeview));	
	gtk_widget_show(GTK_WIDGET(child));		
	
	packages_create_and_fill_model(allpackages, treeview);
	treemodel = gtk_tree_view_get_model(treeview);
	
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
	gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
	
	/* create tablabel */
	GOBJECT_GET_STRING_PROPERTY(query, "searchstring", &searchstring);
	searchtext = (gchar*)g_malloc(sizeof(gchar)*(strlen(searchstring)+10));
	g_sprintf(searchtext, "%s (%d)", searchstring, guyum_package_composite_get_component_count(allpackages)) ;
	
	/* append new page */			
	currentpage = gtk_notebook_insert_page(notebook_query, child, create_tablabel(searchtext, (gpointer*)query), position);		
	gtk_notebook_set_current_page( GTK_NOTEBOOK(glade_xml_get_widget(xml, "guyum_notebook_query")), currentpage);
		
	if (guyum_package_composite_get_component_count(allpackages)>0) {
		g_signal_connect(G_OBJECT(selection), "changed", G_CALLBACK(query_selection_changed), (gpointer)query);		
		/* select first entry in list */
		if (gtk_tree_model_get_iter_first(treemodel, &treepath)) {	
			gtk_tree_selection_select_iter(selection, &treepath);
		}
	}
	g_free(searchstring);
	g_free(searchtext);
}
