/***************************************************************************
 *            repotree.c
 *
 *  Sun May 28 22:32:31 2006
 *  Copyright  2006  User
 *  Email
 ****************************************************************************/

#include <include/ui/ui_repotree.h>

extern GuyumMain* guyummain;
extern GladeXML* xml;
extern GtkWidget* window1;




/**
*
*/
static void
item_toggled (	GtkCellRendererToggle *cell,
	      		gchar                 *path_str,
	      		gpointer               data)
{

	GtkTreeModel *model = (GtkTreeModel *)data;
  	GtkTreePath *sortpath = gtk_tree_path_new_from_string (path_str);
	GtkTreeView *treeview = (GtkTreeView*)glade_xml_get_widget(xml, "guyum_treeview_repo");
	GtkTreeModelSort* sortmodel = (GtkTreeModelSort*)gtk_tree_view_get_model(treeview);
	GtkTreePath *path = gtk_tree_model_sort_convert_path_to_child_path(sortmodel, sortpath);
  	GtkTreeIter iter;
  	gboolean toggle_item;
	gint *column;
	gchar *repoid;

  	column = g_object_get_data (G_OBJECT (cell), "column");
  	
  	/* get toggled iter */
  	gtk_tree_model_get_iter (model, &iter, path);
  	gtk_tree_model_get (model, &iter, column, &toggle_item, -1);

    gtk_tree_model_get(model, &iter, COL_REPOS_ID, &repoid, -1);
	
  	/* do something with the value */
  	toggle_item ^= 1;

  	/* set new value */
  	gtk_tree_store_set (GTK_TREE_STORE (model), &iter, column,
		      toggle_item, -1);

  	/* clean up */
  	gtk_tree_path_free (path);
}


GtkTreeStore*
repos_create_and_fill_model(GuyumRepoComposite* repos, GtkTreeView* treeview)
{
	GtkTreeStore *treestore;
	GtkTreeIter toplevel;
	gint i = 0;
	
	/* repositories treestore */	
	treestore = gtk_tree_store_new(2, G_TYPE_STRING, G_TYPE_BOOLEAN); 
	
	for (i=0; i<guyum_repo_composite_get_component_count(repos); i++) {	
		gchar* idvalue = NULL;
		gboolean enabledvalue ;		

		GuyumRepoComponent* repo = guyum_repo_composite_get_nth_component(repos, i);

		GOBJECT_GET_STRING_PROPERTY(repo, "id", &idvalue);
		GOBJECT_GET_BOOLEAN_PROPERTY(repo, "enabled", &enabledvalue);

		/* append new line to treeview */
        gtk_tree_store_append(treestore, &toplevel, NULL);
		gtk_tree_store_set(treestore, &toplevel, 
							COL_REPOS_ID, idvalue,																			
							COL_REPOS_ENABLED, enabledvalue,							
							-1);

		g_free(idvalue);		
	}

	return treestore;


}


void
create_repos_treemodel(GuyumRepoComposite* repos)
{
	GtkTreeViewColumn 	*col;
	GtkCellRenderer 	*renderer;
    GtkCellRenderer 	*activate_renderer;
	GtkTreeModel 		*model;
	GtkTreeModelSort	*sortmodel;
	GtkTreeView* treeview = (GtkTreeView*)glade_xml_get_widget(xml, 
												"guyum_treeview_repo");
 	if (gtk_tree_view_get_model(GTK_TREE_VIEW(treeview))==NULL) {	
		/* Set rules hint */
		gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(treeview), TRUE);
		/* Column id */
		col = gtk_tree_view_column_new();
		gtk_tree_view_column_set_title(col, "ID");
		gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), col);
		renderer = gtk_cell_renderer_text_new();
		gtk_tree_view_column_pack_start(col, renderer, TRUE);
		gtk_tree_view_column_add_attribute(col, renderer, "text", COL_REPOS_ID);
		
		/* Column Enabled */		
		activate_renderer = gtk_cell_renderer_toggle_new ();
		g_object_set (activate_renderer, "xalign", 0.0, NULL);
		g_object_set_data (G_OBJECT (activate_renderer), "column", (gint *)COL_REPOS_ENABLED);
	
		gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
																			-1, "enabled",
																			activate_renderer,
																			"active",
																			COL_REPOS_ENABLED,
																			NULL);		
		gtk_tree_view_set_headers_clickable(treeview, TRUE);
		model = (GtkTreeModel*)repos_create_and_fill_model(repos, treeview);
		
		sortmodel = (GtkTreeModelSort*)gtk_tree_model_sort_new_with_model(GTK_TREE_MODEL(model));
		gtk_tree_view_set_model(GTK_TREE_VIEW(treeview), GTK_TREE_MODEL(sortmodel));
	
		gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(sortmodel), COL_REPOS_ID, GTK_SORT_ASCENDING);
				
		g_signal_connect (activate_renderer, "toggled", G_CALLBACK (item_toggled), model);	
	}
	
}


void
show_init_repositories()
{
	GuyumRepoComposite* repos;

	GValue pointvalue = {0,};
	g_value_init(&pointvalue, G_TYPE_POINTER);
	
	g_object_get_property(G_OBJECT(guyummain), "repos", &pointvalue);
	
	repos = (GuyumRepoComposite*)g_value_get_pointer(&pointvalue);
	
	create_repos_treemodel(repos);
	
}
